/* osdep.h - defines and macros for reducing platform differences */

/*
 * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
 *               2008, 2009, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadallen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: osdep.h,v 1.8 2010/06/26 00:56:39 karls Exp $ */

#include <sys/types.h>

#ifndef WAIT_ANY
#define WAIT_ANY -1
#endif /* !WAIT_ANY */

#ifdef __sun /*XXX*/
#ifndef NI_MAXHOST
#define NI_MAXHOST 256
#endif
#endif

#ifndef LLONG_MAX
#define LLONG_MAX 9223372036854775807LL
#endif /* !LLONG_MAX */

#ifndef INT32_MAX
#define INT32_MAX 0x7fffffff
#endif /* INT32_MAX */

#ifndef UINT16_MAX
#define UINT16_MAX 0xFFFF
#endif /* UINT16_MAX */

#ifndef BYTE_ORDER

#ifndef LITTLE_ENDIAN
#define LITTLE_ENDIAN 1234
#endif /* LITTLE_ENDIAN */

#ifndef BIG_ENDIAN
#define BIG_ENDIAN 4321
#endif /* BIG_ENDIAN */

#ifdef WORDS_BIGENDIAN /* optionally defined by AC_C_BIGENDIAN */
#define BYTE_ORDER BIG_ENDIAN
#else
#define BYTE_ORDER LITTLE_ENDIAN
#endif /* WORDS_BIGENDIAN */

#endif /* BYTE_ORDER */

#if (defined lint)
#undef __attribute__
#define __attribute__(a)
#endif

#ifdef __sun /* macros defined in strings.h, not string.h */
#define bzero(b,len) (memset((b), '\0', (len)), (void)0)
#define bcopy(b1,b2,len) (memmove((b2), (b1), (len)), (void)0)
#define bcmp(b1,b2,len) memcmp((b1), (b2), (size_t)(len))
#endif /* __sun */

#if !HAVE_DAEMON
int daemon(int nochdir, int noclose);
#endif /* !HAVE_DAEMON */

#if !HAVE_DIFFTIME
double difftime(const time_t time1, const time_t time0);
#endif /* !HAVE_DIFFTIME */

#if !HAVE_INET_PTON
int inet_pton(int af, const char *src, void *dst);
#endif /* !HAVE_INET_PTON */

/* libscompat replacement function prototypes */
#if !HAVE_SETPROCTITLE
void setproctitle(const char *fmt, ...)
   __attribute__((format(__printf__, 1, 2)));
void initsetproctitle(int, char **);
#endif /* !HAVE_SETPROCTITLE */

#if !HAVE_STRLCPY
size_t strlcpy(char *dst, const char *src, size_t size);
size_t strlcat(char *dst, const char *src, size_t size);
#endif /* !HAVE_STRLCPY */

#if !HAVE_STRTONUM
long long strtonum(const char *numstr, long long minval, long long maxval,
		   const char **errstrp);
#endif /* !HAVE_STRTONUM */

#if !HAVE_TIMER_MACROS
#include "tvops.h"
#endif /* !HAVE_TIMER_MACROS */

#if !HAVE_VSYSLOG
void vsyslog(int pri, const char *fmt, va_list ap);
#endif /* !HAVE_VSYSLOG */

#if NEED_GETSOCKOPT_CAST
#define getsockopt(a,b,c,d,e) getsockopt((a),(b),(c),(char *)(d),(e))
#define setsockopt(a,b,c,d,e) setsockopt((a),(b),(c),(char *)(d),(e))
#endif /* NEED_GETSOCKOPT_CAST */

#if !HAVE_SIG_ATOMIC_T
typedef int sig_atomic_t;
#endif /* !HAVE_SIG_ATOMIC_T */

#if SIZEOF_CHAR == 1
 typedef unsigned char ubits_8;
 typedef          char sbits_8;
#else
#error "no known 8 bits wide data type"
#endif

#if SIZEOF_SHORT == 2
 typedef unsigned short ubits_16;
 typedef          short sbits_16;
#else
# if SIZEOF_INT == 2
  typedef unsigned int ubits_16;
  typedef          int sbits_16;
# else
#error "no known 16 bits wide data type"
# endif
#endif

#if SIZEOF_INT == 4
 typedef unsigned int ubits_32;
 typedef          int sbits_32;
#else
# if SIZEOF_SHORT == 4
   typedef unsigned short ubits_32;
   typedef          short sbits_32;
# else
#  if SIZEOF_LONG == 4
    typedef unsigned long ubits_32;
    typedef          long sbits_32;
#  else
#error "no known 32 bits wide data type"
#  endif /* SIZEOF_LONG == 4 */
# endif /* SIZEOF_SHORT == 4 */
#endif /* SIZEOF_INT == 4 */

#if !HAVE_INT8_T
#define int8_t sbits_8
#endif /* HAVE_INT8_T */

#if !HAVE_INT16_T
#define int16_t sbits_16
#endif /* HAVE_INT16_T */

#if !HAVE_INT32_T
#define int32_t sbits_32
#endif /* HAVE_INT32_T */

#if !HAVE_UINT8_T
#define uint8_t ubits_8
#endif /* HAVE_UINT8_T */

#if !HAVE_UINT16_T
#define uint16_t ubits_16
#endif /* HAVE_UINT16_T */

#if !HAVE_UINT32_T
#define uint32_t ubits_32
#endif /* HAVE_UINT32_T */

#if !HAVE_IN_PORT_T
#define in_port_t ubits_16
#endif /* HAVE_IN_PORT_T */

#if !HAVE_IN_ADDR_T
#define in_addr_t ubits_32
#endif /* HAVE_IN_ADDR_T */

#ifndef INADDR_NONE
#define INADDR_NONE (ubits_32)0xffffffff
#endif /* !INADDR_NONE */

#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#endif /* !MAX */

#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif /* !MIN */

#if NEED_SA_RESTART
#define SA_RESTART SV_INTERRUPT
#endif /* NEED_SA_RESTART */
