#compiler related checks, updates CFLAGS and CPPFLAGS and sets
# 'warn' with flags for warnings
#NOTE: save CFLAGS; wish to compile without -O2 when debugging
oCFLAGS=$CFLAGS
unset CFLAGS
AC_AIX #evidently also modifies CFLAGS
AC_PROG_LIBTOOL
autoconf_compflags=$CFLAGS
CFLAGS=$oCFLAGS

AC_PROG_CPP

AC_MSG_CHECKING([for compiler flags])
case $host in
    #XXX check for compiler type, not architecture
    *-*-solaris*)
	if test "x$GCC" = x; then
	    #-xs provides allows easier debugging with gdb
	    comp_flags="-Xt -xs" #XXX -Xt needed to compile?
	    have_suncc=t
	fi
	AC_MSG_RESULT([$comp_flags])
    ;;

    *-*-hpux*)
	if test "x$GCC" = x; then
	    CPPFLAGS="$CPPFLAGS${CPPFLAGS:+ }-D_XOPEN_SOURCE"
	fi
	AC_MSG_RESULT([$comp_flags])
	;;

    *-*-aix4* | *-*-aix5* | *-*-aix6*)
	if test "x$GCC" = x; then
	    comp_flags="-qlanglvl=stdc99:ucs"
	    #XXX
	    CPPFLAGS="$CPPFLAGS${CPPFLAGS:+ }-qlanglvl=stdc99:ucs"
	fi
	AC_MSG_RESULT([$comp_flags])
     ;;

    *)
	AC_MSG_RESULT([none])
    ;;
    #XXX make sure compiling with compiler options works
esac

case $host in
    alpha*-dec-osf*)
	CPPFLAGS="$CPPFLAGS${CPPFLAGS:+ }-D_XOPEN_SOURCE_EXTENDED -DBYTE_ORDER=LITTLE_ENDIAN -D_POSIX_SOURCE -D_POSIX_C_SOURCE=199309L -D_OSF_SOURCE"
	;;

    *-*-solaris*)
	CPPFLAGS="$CPPFLAGS${CPPFLAGS:+ }-D_XOPEN_SOURCE=600 -D_XOPEN_SOURCE_EXTENDED"
	CPPFLAGS="$CPPFLAGS${CPPFLAGS:+ }-D__EXTENSIONS__ -DBSD_COMP"
	;;

    *-*-linux-*)
	CPPFLAGS="$CPPFLAGS${CPPFLAGS:+ }-D_XOPEN_SOURCE=600 -D_XOPEN_SOURCE_EXTENDED"
	CPPFLAGS="$CPPFLAGS${CPPFLAGS:+ }-D_BSD_SOURCE -D_FORTIFY_SOURCE=2"
	;;
esac

AC_MSG_CHECKING([for support for -pipe compiler flag])
oCFLAGS=$CFLAGS
CFLAGS="$CFLAGS${CFLAGS:+ }-pipe"
AC_TRY_RUN([
int main()
{
	return 0;
}], [AC_MSG_RESULT([yes])
     comp_flags="${comp_flags}${comp_flags:+ }-pipe"],
    AC_MSG_RESULT([no]))
CFLAGS="$oCFLAGS"

AC_MSG_CHECKING([for support for -Wbounded compiler flag])
oCFLAGS=$CFLAGS
CFLAGS="$CFLAGS${CFLAGS:+ }-Wbounded"
AC_TRY_RUN([
int main()
{
        return 0;
}], [AC_MSG_RESULT([yes])
     comp_flags="${comp_flags}${comp_flags:+ }-Wbounded"],
    [AC_MSG_RESULT([no])
     AC_DEFINE(__bounded__(a,b,c), , [empty __bounded__ macro])])
CFLAGS="$oCFLAGS"

AC_MSG_CHECKING([for __attribute__ support])
AC_TRY_RUN([
#include <stdlib.h>

void errfunc(void) __attribute((noreturn));

void errfunc(void)
{
    exit(0);
}

int main()
{
    errfunc();
}], [AC_MSG_RESULT([yes])],
    [AC_MSG_RESULT([no])
     AC_DEFINE(__attribute__(a), , [empty __attribute__ macro])])

AC_MSG_CHECKING([for __printf__ attribute support])
if test x"$have_suncc" = xt; then
    AC_MSG_RESULT([disabled for sun cc])
    AC_DEFINE(format(a,b,c), , [empty format attribute macro])
else
    AC_TRY_RUN([
#include <stdlib.h>

void func(const char *fmt, ...)
   __attribute__((format(__printf__, 1, 2)));

void func(const char *fmt, ...) {
     (void)fmt;
     return;
}

int main()
{
    func("foo");
    return 0;
}], [AC_MSG_RESULT([yes])],
    [AC_MSG_RESULT([no])
     AC_DEFINE(format(a,b,c), , [empty format attribute macro])])
fi

AC_MSG_CHECKING([for compilation with debugging])
AC_ARG_ENABLE(debug,
[  --enable-debug          compile with debugging support],
[debug_enabled=t],
[if test x$prerelease != x; then
    debug_enabled=t
 fi])

if test x$debug_enabled = xt; then
    #no optimization wanted
    if test $ac_cv_prog_cc_g = yes; then
	CFLAGS="$CFLAGS${CFLAGS:+ }-g"
    fi
    CPPFLAGS="$CPPFLAGS${CPPFLAGS:+ }-DDEBUG=1"
    AC_MSG_RESULT([yes])
else
    AC_MSG_RESULT([no])
    #autoconf_compflags is set to "-g -O2" with GCC
    #override CFLAGS when running configure to avoid this
    CPPFLAGS="$CPPFLAGS${CPPFLAGS:+ }-DDEBUG=0"
    CFLAGS="$CFLAGS${CFLAGS:+ }$autoconf_compflags"
fi

#-Wall ?
AC_MSG_CHECKING([for warning flags])
AC_ARG_ENABLE(warnings,
[  --enable-warnings       show compilation warnings],
[enable_warnings=t],
[if test x$prerelease != x; then
    enable_warnings=t
 fi])

#construct warning flags in $warn
if test x$enable_warnings != x; then
    #try to enable compiler specific warning flags
    if test "x$GCC" = x; then
	case $host in
	    alpha*-dec-osf*) #osf cc
#		warn="-w0 -check -portable -warnprotos"
		true
	    ;;

	    *-*-hpux*)
		warn="-v"
	    ;;

	    *-*-solaris*)
		warn="-v"
		true
	    ;;

	    *-*-irix*) #sgi cc
		warn="-fullwarn"
	    ;;

	    *) #try -Wall (gcc)
		warn="-Wall"
	    ;;
	esac
    else
	#gcc warnings
	warn="-Wall -Wformat -W -Wnested-externs -Wmissing-declarations -Wmissing-prototypes -Wstrict-prototypes -Wcast-align -Wcast-qual -Wbad-function-cast -Wpointer-arith -Wundef"
	#warn="$warn -Wold-style-cast -Winline -Waggregate-return -Wconversion -Wwrite-strings -Wtraditional -Wshadow"
    fi

    oCFLAGS=$CFLAGS
    CFLAGS="$CFLAGS${CFLAGS:+ }$warn"
    #make sure compilation is still possible
    AC_TRY_COMPILE([], [],
		   [AC_MSG_RESULT([$warn])],
		   [AC_MSG_RESULT([none])
		    unset warn])
    CFLAGS=$oCFLAGS
else
    AC_MSG_RESULT([none])
fi

#NOTE: set warnings at the bottom; might interfere with tests
CFLAGS="$CFLAGS${CFLAGS:+ }$comp_flags"

#-DDIAGNOSTICS?
AC_MSG_CHECKING([for compliation with DIAGNOSTIC])
AH_TEMPLATE([DIAGNOSTIC], [for debugging])
AC_ARG_ENABLE(diagnostic,
[  --enable-diagnostic     enable diagnostic],
[AC_DEFINE(DIAGNOSTIC, 1)
 AC_MSG_RESULT([yes])],
[if test x$prerelease != x; then
    AC_DEFINE(DIAGNOSTIC, 1)
    AC_MSG_RESULT([yes])
 else
    AC_DEFINE(DIAGNOSTIC, 0)
    AC_MSG_RESULT([no])
 fi])

AC_MSG_CHECKING([for profiled compilation])
AC_ARG_ENABLE(profiling,
[  --enable-profiling      compile with profiling support in server],
[AC_DEFINE(HAVE_PROFILING, 1, [for profiling])
 LDFLAGS="$LDFLAGS${LDFLAGS:+ }-pg"
 CFLAGS="$CFLAGS${CFLAGS:+ }-pg -DPROFILING"
 AC_MSG_RESULT([yes])

 case $host in
     *-*-openbsd* | *-*-freebsd*)
         #static linking, disable server preloading
      	 no_preload_server=t
	 #openbsd and freebsd appear to be happier if -lc is included
	 #when profiling is enabled
	 LIBS="$LIBS${LIBS:+ }-lc"
	 ;;
 esac],
[AC_MSG_RESULT([no])])

AC_ARG_ENABLE(linting,
[  --enable-linting        enable lint],
[AC_CHECK_PROG(LINT, lint, lint)
 if test x"$LINT" = x; then
     AC_MSG_WARN([linting requested, but lint not found])
 else
     s_linting=t
     LINTFLAGS=-abcebiprxz
 fi])
AM_CONDITIONAL(RUNLINT, test x$s_linting = xt)
AC_SUBST(LINT)
AC_SUBST(LINTFLAGS)
