dnl libscompat.m4 - tests related to replacement code in libscompat directory

#SO_SNDLOWAT
#irix 6.2 only seems to look at lowest 8 bit of value
#Solaris also seems to be broken (up to 2.6 atleast)
AC_MSG_CHECKING([whether a working SO_SNDLOWAT exists])
AC_EGREP_CPP(yes,
[#include <sys/socket.h>
#ifdef SO_SNDLOWAT
yes
#endif],
[case $host in
    *bsd*)
	so_sndlowat=t
	;;

    *-osf*)
	so_sndlowat=t
	;;
esac
if test "x${so_sndlowat}" = xt; then
    AC_DEFINE(HAVE_SO_SNDLOWAT, 1, [platform supports SO_SNDLOWAT])
    AC_MSG_RESULT(yes)
else
    AC_MSG_RESULT(no)
fi], [AC_MSG_RESULT(no)])


case $host in
    *-*-linux-*)
	AC_DEFINE(SPT_TYPE, SPT_REUSEARGV, [setproctitle replacement type])
	;;
esac

dnl Checking variable sizes                                                     
AC_CHECK_SIZEOF(char)
AC_CHECK_SIZEOF(short)
AC_CHECK_SIZEOF(int)
AC_CHECK_SIZEOF(long)

AC_C_BIGENDIAN

AC_CHECK_HEADERS(bitstring.h sys/pstat.h)

AC_SYS_LARGEFILE

AC_MSG_CHECKING([if getsockopt needs cast])
AC_TRY_COMPILE([
#include <sys/types.h>
#include <sys/socket.h>
int getsockopt(int, int, int, char *, int *);
], [ 1 ],
 [AC_DEFINE(NEED_GETSOCKOPT_CAST, 1, [getsockopt needs cast])
  AC_MSG_RESULT(yes)],
  AC_MSG_RESULT(no))

#XXX should be more generic, check if nonexistent
#SV_INTERRUPT, but not SA_RESTART defined?
AC_MSG_CHECKING([to see if SV_INTERRUPT should be used])
AC_EGREP_CPP(yes, [
#include <signal.h>
#ifdef SA_RESTART
#else
# ifdef SV_INTERRUPT
yes
# endif
#endif
], [AC_DEFINE(NEED_SA_RESTART, 1, [use SA_RESTART, not SV_INTERRUPT])
    AC_MSG_RESULT(yes)],
    AC_MSG_RESULT(no))

#XXXseems to be supported on Solaris2.6, but there might be some
#defines that need to be set (should _XOPEN_SOURCE be defined on all
#platforms?)
AC_MSG_CHECKING([if cmsghdr exists in <sys/socket.h>])
case $host in
    *)
	AC_TRY_COMPILE([
#include <sys/types.h>
#include <sys/socket.h>
], [struct cmsghdr foo = {1,1,1};
 struct msghdr bar;
 foo.cmsg_len = 0;
 foo.cmsg_type = SCM_RIGHTS;
 foo.cmsg_level = SOL_SOCKET;
 bar.msg_controllen = 1;
], [AC_DEFINE(HAVE_CMSGHDR, 1, [struct cmsghdr exists])
    AC_MSG_RESULT(yes)],
    AC_MSG_RESULT(no))
	;;
esac

AC_MSG_CHECKING([for CMSG_SPACE in sys/socket.h])
AC_TRY_RUN([
#include <sys/types.h>
#include <sys/socket.h>
int main()
{
   int d = CMSG_SPACE(4);
   return 0;
}
], [AC_MSG_RESULT(yes)
    AC_DEFINE(HAVE_CMSG_SPACE, 1, CMSG_SPACE exists)],
   [AC_MSG_RESULT(no)])

AC_MSG_CHECKING([for CMSG_LEN in sys/socket.h])
AC_TRY_RUN([
#include <sys/types.h>
#include <sys/socket.h>

int main()
{
   int d = CMSG_LEN(4);
   return 0;
}], [AC_MSG_RESULT(yes)
    AC_DEFINE(HAVE_CMSG_LEN, 1, [CMSG_LEN exists])],
   [AC_MSG_RESULT(no)])

AC_MSG_CHECKING([for sa_len in sockaddr])
AC_TRY_COMPILE([
#include <sys/types.h>
#include <sys/socket.h>
], [struct sockaddr foo;
foo.sa_len = 0;
], [AC_DEFINE(HAVE_SOCKADDR_SA_LEN, 1, [sa_len exists in sockaddr])
    AC_MSG_RESULT(yes)],
   [AC_MSG_RESULT(no)])

#AIX 6.1 needs sys/select.h, but can be problematic on other platforms
AC_MSG_CHECKING([is sys/select.h is needed])
AC_TRY_COMPILE([
#include <sys/types.h>
#include <sys/time.h>
#include <string.h>
#include <unistd.h>
], [fd_set *fdp;
    size_t p;
    fdp = NULL;
    p = NFDBITS * sizeof(fd_mask);
], [AC_MSG_RESULT(no)],
   [dnl compilation failure, try with sys/select.h (ignore if this fails)
    AC_TRY_COMPILE([
#include <sys/types.h>
#include <sys/time.h>
#include <sys/select.h>
#include <string.h>
#include <unistd.h>
], [fd_set *fdp;
    size_t p;
    fdp = NULL;
    p = NFDBITS * sizeof(fd_mask);],
    [AC_DEFINE(HAVE_SYS_SELECT_H, 1, [sys/select.h needed])
     AC_MSG_RESULT(yes)])])

AC_MSG_CHECKING([to see if malloc_options exists])
AC_TRY_LINK([extern char *malloc_options;],
[ malloc_options = 0; ],
[AC_DEFINE(HAVE_MALLOC_OPTIONS, 1, [support for malloc debugging])
 AC_MSG_RESULT(yes)],
[AC_MSG_RESULT(no)])

AC_MSG_CHECKING([to see if __progname exists])
AC_TRY_LINK([extern char *__progname;],
[ __progname = 0; ],
[AC_DEFINE(HAVE_PROGNAME, 1, [programe name symbol exists])
 AC_MSG_RESULT(yes)],
[AC_MSG_RESULT(no)])

#FreeBSD has setproctitle in -lutil (Per Hedeland <per@erix.ericsson.se>)
AC_SEARCH_LIBS(setproctitle, util)

#expected select behaviour?
#unset nb_select_err
#L_UNCON_SELECT([],
# [nb_select_err=t])
#if test x"${nb_select_err}" = xt; then
#   AC_MSG_WARN([operations on nonblocking sockets might fail on this platform])
#fi

#Linux (redhat 5.2) defines socklen_t in <socketbits.h>, which is
#included by <sys/socket.h>.  check for this first.
AC_MSG_CHECKING([for socklen_t])
AC_TRY_COMPILE([
#include <sys/types.h>
#include <sys/socket.h>
], [socklen_t foo = 1;],
   [AC_MSG_RESULT(yes)
    socklen_found=t],
   [AC_MSG_RESULT(no)
    socklen_found=""])

AH_TEMPLATE([socklen_t], [platform workaround])
if test x"$socklen_found" = x; then
    case $host in
	alpha*-dec-osf* | *-*-aix*)
	    AC_DEFINE(socklen_t, size_t)
	    ;;

	*)
	    AC_DEFINE(socklen_t, int)
	    ;;
    esac
fi

#sig_atomic_t
AC_MSG_CHECKING([for sig_atomic_t in <signal.h>])
AC_EGREP_CPP(sig_atomic_t, [
#include <signal.h>
], [AC_DEFINE(HAVE_SIG_ATOMIC_T, 1, [sig_atomic_t defined in signal.h])
    case $host in
	*-*-aix*)
	    AC_DEFINE(HAVE_VOLATILE_SIG_ATOMIC_T, 1, [platform workaround])
	;;
    esac
    AC_MSG_RESULT(yes)],
   [AC_MSG_RESULT(no)])

AC_CHECK_TYPES([int8_t, int16_t, int32_t, uint8_t, uint16_t, uint32_t,
		in_port_t, in_addr_t],
		, ,
[
#include <sys/types.h>
#include <netinet/in.h>
])
AC_CHECK_TYPE(ssize_t, int)

dnl Checks for library functions.
AC_PROG_GCC_TRADITIONAL
AC_FUNC_MEMCMP
AC_TYPE_SIGNAL
AC_FUNC_STRFTIME
AC_FUNC_VPRINTF

AC_MSG_CHECKING([for queue macros])
AC_TRY_RUN([
#include <sys/queue.h>

SLIST_HEAD(connections, connection);
SLIST_ENTRY(connection) n;

int main()
{
    return 0; }],
[AC_MSG_RESULT(yes)
 AC_DEFINE(HAVE_QUEUE_MACROS, 1, [SLIST macros exist in sys/queue.h])],
[AC_MSG_RESULT(no)])

AC_MSG_CHECKING([for timer macros])
AC_TRY_RUN([
#include <sys/time.h>

int main()
{
    struct timeval tv, tv2, tv3;

    tv.tv_sec = 0;
    tv.tv_usec = 0;
    tv2.tv_sec = 0;
    tv2.tv_usec = 0;
    tv3.tv_sec = 0;
    tv3.tv_usec = 0;

    timeradd(&tv, &tv2, &tv3);
    timersub(&tv3, &tv2, &tv);

    return 0; }],
[AC_MSG_RESULT(yes)
 AC_DEFINE(HAVE_TIMER_MACROS, 1, [timeradd(), etc. exist in sys/time.h])],
[AC_MSG_RESULT(no)])

#sendfile XXX only linux variant supported
case $host in
    *-*-linux-*)
        AC_CHECK_HEADERS(sys/sendfile.h)
	AC_CHECK_FUNCS(sendfile)
	;;
esac

#solaris libraries
AC_SEARCH_LIBS(connect, socket)
AC_SEARCH_LIBS(inet_pton, nsl)
AC_SEARCH_LIBS(fabs, m)

#functions with identically named libscompat replacement source file
AC_CHECK_FUNCS(daemon difftime inet_pton setproctitle strlcpy strtonum)
AC_CHECK_FUNCS(strvis vsyslog)
#functions that require special handling
AC_CHECK_FUNCS(bzero MD5Init RMD160Init)

#only compile files that are needed
unset LIBSCSRC
for func in daemon difftime inet_pton setproctitle strlcpy strtonum \
            strvis vsyslog; do
    var=ac_cv_func_${func}
    if test ! -s "libscompat/${func}.c"; then
	AC_MSG_WARN([error: libscompat file for $func missing])
	exit 1
    fi
    if eval "test x\"\$${var}\" = xno"; then
	LIBSCSRC="${LIBSCSRC}${LIBSCSRC:+ }${func}.lo"
    fi
done

if test x"${ac_cv_func_bzero}" = xno; then
    AC_DEFINE(bzero(b, len), memset((b), 0, (len)), [bzero replacement])
fi

if test x"$SSLdigest" != xyes; then
    if test x"${ac_cv_func_MD5Init}" = xno; then
    	LIBSCSRC="${LIBSCSRC}${LIBSCSRC:+ }md5.lo"
    fi
    if test x"${ac_cv_func_RMD160Init}" = xno; then
    	LIBSCSRC="${LIBSCSRC}${LIBSCSRC:+ }rmd160.lo"
    fi
fi

AC_SUBST([LIBSCSRC])
