/* dlagent.h - download agent defines and data structures */

/*
 * Copyright (c) 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: dlagent.h,v 1.10 2010/06/30 08:37:59 karls Exp $ */

#include <sys/queue.h>

#include "aqueue.h"

enum dlstate {
	DLAGENT_STATE_INACTIVE = 0,
	DLAGENT_STATE_PEERS,
	DLAGENT_STATE_GET,
	DLAGENT_STATE_COMPLETE,
	DLAGENT_STATE_MULTFAIL,
};

#define RETRY_GET_MAX 5

struct dlagent {
#ifdef notyet
	int type; /* block retrieval: sequential|random|least-freq */
#endif
	struct urient *ue;
	/* goal is to retrieve blocks in this range sequentially */
	off_t start;
	off_t end;
	int32_t currblock;

	struct timeout *to;
	aqueue_t *readers; /* head wait queue */
	enum dlstate state;
	int retries;
	int refcnt;
	int stopcnt; /* number of stopped readers (due to no data) */

	SLIST_ENTRY(dlagent) n;
};

SLIST_HEAD(dlagents, dlagent);

struct sppconn;

struct dlagent *dlagent_search(struct urient *, off_t, off_t);
void dlagent_addclient(struct dlagent *, struct sppconn *);
void dlagent_delclient(struct dlagent *, struct sppconn *);
void dlagent_purgedblock(struct urient *, int32_t);
void dlagent_cmdok(struct dlagent *, struct sppconn *);
void dlagent_sendok(struct sppconn *, int32_t);
void dlagent_cmdfail(struct dlagent *, struct sppconn *);
void dlagent_tooslow(struct dlagent *);
void dlagent_streamstop(struct dlagent *, struct sppconn *);
void dlagent_timeout(void *);
void dlagent_connend(struct sppconn *);
void dlagent_addref(struct dlagent *);
void dlagent_unref(struct dlagent *);
void dlagent_readdata(struct dlagent *, struct sppconn *);
void dlagent_getcomp(struct dlagent *, struct sppconn *);
void dlagent_status(void);
char *dlagent_state2str(enum dlstate);
