/* filter.c - cache uri filtering allowing limits on requested files */

/*
 * Copyright (c) 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: filter.c,v 1.1 2010/06/29 08:53:45 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <ctype.h>
#include <string.h>

#include "p2pd.h"
#include "alist.h"
#include "filter.h"
#include "uri.h"

/* XXX functionality very basic */

/*
 * the filter has two modes of operation:
 *
 * 'pass all' - all uris are accepted (default)
 * 'pass allowed' - only explicitly listed uris are allowed
 *
 *  adding a host with filter_passhost() enables the second mode
 */

int filter_passall = 1; /* default is pass all */

struct alist *al_filter;


/* filter_preinit - init filter db */
void
filter_preinit(void)
{
	/* start at size zero to use as little memory as possible */
	al_filter = alist_init(NULL, NULL, 0, ALIST_SIZE_GROW, ALIST_TYPE_STR);
}


/* filter_passhost - enable filter and allow access to files at host */
int
filter_passhost(char *host)
{
	const char *f = "filter_passhost";
	char lchost[NI_MAXHOST];
	int i;

	filter_passall = 0;

	if (strlen(host) + 1 > sizeof(lchost)) {
		swarn("%s: host too long", f);
		return -1;
	}

	/* normalize; parsed uris are lowercase */
	for (i = 0; (size_t)i < strlen(host); i++)
		lchost[i] = (char)tolower(host[i]);
	lchost[i] = '\0';

	if (alist_find(al_filter, lchost) != NULL) {
		swarn("%s: host already exists: %s", f, host);
		return -1;
	}
	/* only interested in key, add dummy value */
	if (alist_add(al_filter, lchost, (void *)0xdeadbeef) == NULL)
		ERRX("alist_add failed"); /* should not occur for GROW array */

	return 0;
}


/* filter_checkuri - return 0 if access to uri is allowed, -1 otherwise */
int
filter_checkuri(struct uri *uri)
{
	if (filter_passall)
		return 0; /* pass all */

	if (alist_find(al_filter, uri->host) != NULL)
		return 0;
	else
		return -1;
}
