/* mkdigest.c - standalone .sum file creation */

/*
 * Copyright (c) 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: mkdigest.c,v 1.10 2010/07/20 19:17:51 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <err.h>
#include <stdio.h>
#include <string.h>

#include "p2pd.h"
#include "block.h"
#include "blocksum.h"
#include "file.h"
#include "memcache.h"
#include "mkdigest.h"
#include "servfile.h"
#include "spp.h"
#include "sumfile.h"
#include "uri.h"


/* digest_file - create full .sum file for file given as parameter */
void
digest_file(char *path)
{
	char fakeuri[URI_MAXLEN];
	struct urient *ue;
	struct uri uri;
	int32_t i;

	/*
	 * this function is used outside the normal system operation,
	 * so some manual work is needed.
	 */

	serversumfiles = 1;

	/* fake an uri */
	strlcpy(fakeuri, SCHEME_SPP, sizeof(fakeuri));
	strlcat(fakeuri, "://localhost/", sizeof(fakeuri));
	if (strlcat(fakeuri, path, sizeof(fakeuri)) >= sizeof(fakeuri))
		err(1, "filename too long");
	if (uri_parse(&uri, fakeuri) == -1)
		err(1, "filename processing error");

	dbg("uri: %s", uri.uri);
	if ((ue = urient_servinit(&uri)) == NULL ||
	    urient_fileref(ue) == -1)
		err(1, "file init failure");

	/* generate digest for each block */
	for (i = 0; i < ue->meta->blockcnt; i++)
		block_gendigest(ue, i);

	urient_fileunref(ue);
	urient_del(ue);

	fprintf(stdout, " %s, %d blocks\n", path, i);
}
