/* p2pd-mdgen.c - file block sum generation tool */

/*
 * Copyright (c) 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: p2pd-mdgen.c,v 1.5 2010/10/06 08:35:48 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <err.h>
#include <unistd.h>

#include "p2pd.h"
#include "block.h"
#include "digest.h"
#include "memcache.h"
#include "mkdigest.h"
#include "servfile.h"
#include "urient.h"

extern int debug;
/*XXX*/
extern char *pubdir;
uint32_t blocksize_default;

/* XXX */
int igot_sslonly;

void usage(void);


/* usage - print p2pd usage string and exit */
void
usage(void)
{
	errx(1, "usage: p2pd-mdgen [-d] [-b block size] [-m digest] <file(s)>");
}


/* main - it all starts here */
int
main(int argc, char *argv[])
{
	const char *errstr;
	uint32_t blocksize;
	char *mdname;
	int ch;
	int i;

	urient_preinit();
	log_preinit("p2pd-mdgen", log_stderr, NULL);
/*	peer_preinit(peer_opsinit, peer_opsfree);*/
/*	queue_preinit();*/

	opterr = 0;

	debug = 0;
	blocksize = BLOCKSIZE_DEFAULT;
	mdname = DIGEST_DEFAULT;

	while ((ch = getopt(argc, argv, "b:dm:")) != -1) {
		switch (ch) {
		case 'b':
			blocksize = strtonum(optarg, 0, FILESIZE_MAX, &errstr);
			if (errstr || !block_validsize(blocksize))
				errx(1, "blocksize: %s", errstr);
			break;

		case 'd':
			debug = 1;
			break;

		case 'm': /* set default message digest */
			mdname = optarg;
			break;

		default:
			usage();
		}
	}
	argc -= optind;
	argv += optind;

#if 0
	if (servfile_preinit(blocksize, "") == -1)
		errx(1, "server startup failed");
#else
	blocksize_default = blocksize;
	pubdir = "";
#endif
	memcache_preinit(0);
	digest_preinit(mdname);
	fprintf(stderr, "using %d block len %s digests\n", blocksize, mdname);
	for (i = 0; i < argc; i++)
		digest_file(argv[i]);
	exit(0);
}
