/* parse_peers.c - PEERS response parsing function */

/*
 * Copyright (c) 2007, 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: parse_peers.c,v 1.13 2010/06/30 08:37:59 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <string.h>

#include "p2pd.h"
#include "spp.h"


/* peers_parse - PEERS response data parsing */

int
peers_parse(struct peerline *pline, int size, char *buf)
{
	const char *f = "peers_parse";
	char *addr, *port, *bcnt;
	const char *errstr;
	size_t len;
	char *line;
	char *e;
	int i;

	ASSERT(size > 0, "invalid array size");

/*	dbg("%s: %s", f, buf);*/

	i = 0;
	line = buf;
	for (;;) {
		/* end of data if no terminating newline */
		if ((e = strstr(line, "\r\n")) == NULL)
			break;
		*e = '\0';

		if (i >= size) {
			dbg("%s: too many peer entries, ignoring", f);
			break;
		}

		addr = line;

		if ((port = strchr(addr, ' ')) != NULL) {
			*port = '\0';
			port++;
		} else {
			dbg("%s: peer port number missing", f);
			return -1;
		}

		if ((bcnt = strchr(port, ' ')) != NULL) {
			*bcnt = '\0';
			bcnt++;
		}

		/* host */
		len = sizeof(pline[i].addr);
		if (strlcpy(pline[i].addr, addr, len) >= len) {
			dbg("%s: peer address too long", f);
			return -1;
		}

		/* port */
		len = sizeof(pline[i].port);
		if (strlcpy(pline[i].port, port, len) >= len) {
			dbg("%s: peer port value too long", f);
			return -1;
		}

		/* bcnt */
		if (bcnt != NULL) {
			pline[i].cnt = strtonum(bcnt, 0, FILEBLOCKS_MAX,
						&errstr);
			if (errstr) {
				dbg("%s: block count: %s", f, errstr);
				return -1;
			}
		}

		line = e + 2;

		i++;
	}

	if (i == 0) {
		dbg("%s: no peer entries found", f);
		return -1;
	} else
		return i;
}
