/* scc.c - scc related operations */

/*
 * Copyright (c) 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: scc.c,v 1.8 2010/07/17 18:14:04 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <netdb.h>
#include <string.h>

#include "p2pd.h"
#include "scc.h"
#include "spp.h"

/* XXX won't work with ipv6 addresses, use separate arguments instead? */

char parent_name[NI_MAXHOST]; /* SCC proxy name/address */
char parent_port[NI_MAXSERV]; /* SCC proxy port */
int parent_scc = 0; /* set if parent node is defined */


/* scc_parsearg - parse scc parent node address specifier (adr:port) */
int
scc_parsearg(char *adr)
{
	const char *f = "scc_parsearg";
	size_t maxlen;
	char *host;
	char *port;
	char *p;

	host = adr;
	if ((p = strchr(host, ':')) == NULL) {
		port = SPP_PORT;
	} else {
		if (p == host)
			return -1; /* nothing before ':' */
		if (strlen(p) == 1)
			return -1; /* nothing after ':' */
		*p = '\0';
		p++; /* point at data beyond ':' */
		port = p;
	}

	maxlen = sizeof(parent_name);
	if (strlcpy(parent_name, host, maxlen) >= maxlen) {
		swarn("%s: parent name too long: %s", f, host);
		return -1;
	}

	maxlen = sizeof(parent_port);
	if (strlcpy(parent_port, port, maxlen) >= maxlen) {
		swarn("%s: parent port too long: %s", f, port);
		return -1;
	}

	parent_scc = 1;

	return 0;
}
