/* spp.h - primary header for spp code, defines, structures, prototypes */

/*
 * Copyright (c) 2007, 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: spp.h,v 1.74 2010/10/11 17:44:38 karls Exp $ */

#ifndef _SPP_H_
#define _SPP_H_

#include "aqueue.h"
#include "conn.h"
#include "cred.h"
#include "urient.h"

#define RETRY_DELAY_HEAD 3 /* seconds */
#define RETRIES_MAX_HEAD 5
#define RETRY_DELAY_PEERS 5 /* seconds */
/*#define RETRIES_MAX_HEAD 3*/
#define RETRY_DELAY_GET 5 /* seconds */

#define NEWLINE "\r\n"

#define COMMAND_CRED "CRED "
#define COMMAND_GET "GET "
#define COMMAND_HEAD "HEAD "
#define COMMAND_IGOT "IGOT "
#define COMMAND_PEERS "PEERS "
#define COMMAND_PING "PING "
#define COMMAND_REVCONN "REVCONN "
#define COMMAND_STARTTLS "STARTTLS "
#define PROTNAME_SPP "SPP"
#define PROTNAME_HTTP "HTTP"
#define VALUE_VERSION_SPP PROTNAME_SPP "/1.0"
#define VALUE_VERSION_HTTP10 PROTNAME_HTTP "/1.0"
#define VALUE_VERSION_HTTP11 PROTNAME_HTTP "/1.1"
#define OPTION_AGENT "User-Agent: "
#define OPTION_BINDPORT "Bind-Port: " /* XXX should be addr + port */
#define OPTION_BLOCKSIZE "Blocksize: "
#define OPTION_CONNCLOSE "Connection: close"
#define OPTION_CONNECTION "Connection: "
#define OPTION_CONTLEN "Content-Length: "
#define OPTION_DBREFRESH "Block-Info: resend-all" /* resend igot */
#define OPTION_HOST "Host: "
#define OPTION_RANGE "Range: "
#define OPTION_RTT "RTT: "
#define OPTION_NOUNCE "Nounce: "
#define OPTION_SUBSEQ "Subsequent-Blocks: "
#define PARAM_BYTERANGE "bytes="
/* XXX PARAM_BLOCKRANGE "blocks=a-b" | "blocks=a" */

enum sppcmd {
	CLIENT_CMD_CRED = 1,
	CLIENT_CMD_GET,
	CLIENT_CMD_HEAD,
	CLIENT_CMD_IGOT,
	CLIENT_CMD_PEERS,
	CLIENT_CMD_PING,
	CLIENT_CMD_STARTTLS,
	CLIENT_CMD_REVCONN,
};

/* for keeping track of most useful sources of information */
#define BLOCKINFO_IGOT "IGOT"
#define BLOCKINFO_CRED "IGOT credit"
#define BLOCKINFO_PEERS "PEERS"
#define BLOCKINFO_REVCONN "REVCONN"
#define BLOCKINFO_GET "GET"
#define BLOCKINFO_PING "PING"

#define NEWLINE "\r\n"

/* max number of entries in a PEERS reply */
#define MAX_REDIRECT 50 /*XXX*/

/* do at least this many ping tests after receiving revconn */
#define PEER_REVCONN_PINGCNT 3

/*
 * a peer which downloads more than this value, without uploading is
 * marked as a leecher
 */
#define PEER_LEECHSCORE_MAXBLOCKS 20

struct req; /*XXX*/

enum sppconn_type {
	SPPCONN_INCOMING = 1,
	SPPCONN_OUTGOING,
	SPPCONN_RESOLVE,
	SPPCONN_MCONN,
};

/* upper limit on entries in resolve queue to allow head request */
#define SPP_HEAD_MAXRESQUEUE 2

/* max number of active outgoing head requests */
#define SPP_HEAD_OUTGOING_MAX 5

#define SPP_DBREFRESH_INTERVAL 120

#define SPP_TIMEOUT_UNUSED (60 * 60)
#define SPP_TIMEOUT_ERR 10

/* base value, random length added */
#define SPP_NEWPEER_TIMEOUT 10
/* minimum delay between cred attempts */
#define SPP_CRED_MINDELAY 10

/* time before we start looking for potential downloaders */
#define FILE_PROACTIVEREQ_DELAY 60
#define FILE_PROACTIVEREQ_MINBEHIND 10

/* wait time between revconn attempts */
#define PEER_REVCONN_INTERVAL 60

#define SPP_NOUNCE_MAX 65536 /* for ping command */

enum sppstate {
	SPP_STATE_UNUSED,
	SPP_STATE_ACTIVE,
	SPP_STATE_ERR,
};

struct sppfile {
	int sentigot; /* at least one igot successfully submitted */
	enum sppstate state;
	int master; /* true if this is master copy of file (not cached) */
	aqueue_t *waiting; /* clients waiting for head result */

	struct peer *parent;

	int headactive; /* head request ongoing */
	int head_attempt;
	struct timeout *to_head; /* head attempt timeout */

	time_t lastcredcheck; /* time of last cred request, request sent */
	time_t lastcredres; /* time of last cred request, reply received */

	time_t lastrevconn; /* time of previous revconn attempt */

	/* XXX currently not used */
	time_t laststreamreq; /* time of last received stream request */
	time_t lastreq; /* time of last request */
	time_t lastpeers; /* time of last peers request */
/*	float minrtt; */ /*XXX here? *//* closest peer observed for file */

	time_t lastrefresh; /* time of last sending of igots after dbrefresh */

	struct timeout *to;
	struct timeout *to_revconn; /* revconn of cached files timer */

	/*
	 * XXX need some way to indicate that status changed at server,
	 *     e.g., file was ok, then deleted at server, causing requests
	 *     to return errors.
	 *     two main ways to handle:
	 *      1: delete file at once
	 *      2: stop contacting server about file, but let it lie in cache
	 */
	/* errors, let lower layer (urient) handle? */
	int spp_err; /* contains SPP error response code in case of error */
	int refcnt;
	struct urient *ue;
};

/* UTOSF/UETOSF? XXX*/
#define UTOSPP(ue) ((struct sppfile *)(ue)->uldat)
#define SPPTOU(spp) ((spp)->ue)

struct sppconn {
	int httpreq; /* received as http request */
	int sccclient; /* received from scc client */
	int cachereq; /* XXX? request from cache */
	int streamreq; /* stream request (i.e., not block request) */
	struct urient *ue;
	struct dlagent *dl;
	unsigned int dlblock; /*XXX*/
	off_t dlstart;
	off_t dlend;
	enum sppcmd currcmd; /* current spp command */
	int headwait; /* XXX on head wait queue */
	int stopped; /* stream stopped */
	int sppcode;
	int32_t nounce; /* nounce included in ping request */
	struct req *req; /* non-NULL for connections in HEAD chain */
	struct connection *conn; /* underlying connection */
	struct peer *sslinitpeer; /* peer to which sslinit is being done */
};

#define CTOSC(c) ((struct sppconn *)(c)->uldat)
#define SCTOC(sc) ((sc)->conn)

struct peerline {
	char addr[NI_MAXHOST];
	char port[NI_MAXSERV];
	int32_t cnt;
	float rttdist;
};

struct rep {
	int code;
	off_t size;
	int32_t bcnt;
	int have_nounce;
	int32_t nounce;
	int dbrefresh;
	uint32_t blocksize; /* file blocksize */
};

struct speer {
	struct pcred pcred[PEER_CREDLEN];
	unsigned int pcredlen;

	int revconn; /* reverse ping count after revconn received */

	int64_t leechscore; /* scc/server value */
	off_t unreported; /* uncredited data for this peer (received) */
	off_t uncredited; /* uncredited data for this peer (transmitted) */

	int sccclient; /* client of scc node */

	struct timeout *to_newpeer; /* timeout for contacting new peers */
	struct urient *ue_newpeer; /* ue to use for newpeer ping */

	/* server only */ /*XXX free if nodes ever removed */
	int sentrecov; /* recovery indication sent to client */
	struct timeout *to_igot; /* timeout for sending igot to server */
	struct alist *al_igots; /* outstanding igots to server */

	int igotactive; /* igot request active */
	int sslinitactive; /* starttls request ongoing */

	time_t prevcred; /* time of last issued cred command */

	struct peer *peer; /* underlying peer structure */
};

#define SPTOP(sp) ((sp)->peer)
#define PTOSP(p) ((struct speer *)(p)->uldat[PROTO_SPP])

extern int errno_spp;
extern char *lport;

/* spp */
extern struct protops proto_spp;

struct peer *spp_parent(struct uri *);
void spp_parentreq(struct sppconn *);
void spp_peerreq(struct sppconn *, struct peer *);
struct sppconn *sppconn_new(enum sppcmd);
void sppconn_setconn(struct sppconn *, struct connection *);
void sppconn_setfile(struct sppconn *, struct urient *);
int spp_extclient(struct sppconn *);
char *spp_cmd2str(enum sppcmd);

/* speer */
struct speer *speer_add(char *, char *);
void speer_free(struct speer *);
void speer_wantinfo(struct urient *, struct speer *);
void speer_gettinginfo(struct speer *);

/* sppfile */
struct sppfile *sppfile_new(struct urient *);
void sppfile_free(void *);
void sppfile_addref(struct sppfile *);
void sppfile_unref(struct sppfile *);
struct sppfile *sppfile_clientreq(struct uri *);
struct urient *sppfile_servinit(struct uri *);
struct urient *sppfile_headinit1(struct uri *);
void sppfile_cacheinit(struct sppfile *);
void sppfile_seterr(struct sppfile *, int);
void sppfile_timersync(struct sppfile *);
void sppfile_timeout(struct sppfile *);
void sppfile_connfail(struct sppfile *, struct sppconn *);

/* sppfile_cred */
void sppfile_wantcred(struct urient *);

/* sppfile_head */
void sppfile_headinit(struct sppfile *);
void sppfile_waithead(struct sppfile *, struct sppconn *);
void sppfile_headfail(struct sppfile *, struct sppconn *, int);
void sppfile_headok(struct sppfile *, struct sppconn *);
void handle_headchain(struct sppfile *);
void sppfile_tryhead(struct sppfile *);

/* sppfile_revconn */
void sppfile_revconninit(void);
void revconn_timerupd(void);
void revconn_anyfile(void *);

/* spp_cred */
void spp_cmd_cred(struct sppconn *);
void handle_cred(struct sppconn *, struct req *);
int parse_hostcred(struct sppconn *);
void handleresp_cred(struct rep *, struct sppconn *);

/* spp_get */
int spp_cmd_get(struct sppconn *, off_t, off_t);
int handle_get(struct sppconn *, struct req *);
void handleresp_get(struct rep *, struct sppconn *);
void sppget_postrep(struct sppconn *);

/* spp_head */
int spp_cmd_head(struct sppconn *);
void handle_head(struct sppconn *, struct req *);
void handleresp_head(struct rep *, struct sppconn *);

/* spp_igot */
void spp_cmd_igot(struct sppconn *);
void handleresp_igot(struct rep *, struct sppconn *);
void igot_add(struct urient *, int32_t, char *, struct peer *);
void parse_igoterrs(struct sppconn *);
void igot_flushnode(struct peer *);
void dbrefresh(struct urient *);
void dbrefresh_all(void);
void igot_restore(struct urient *);
int handle_igot(struct sppconn *, struct req *);
int parse_igots(struct sppconn *);

/* spp_peers */
int spp_cmd_peers(struct sppconn *, int32_t);
void handleresp_peers(struct rep *, struct sppconn *);
void parse_peerlist(struct sppconn *);
void handle_peers(struct sppconn *, struct req *);

/* parse_peers */
int peers_parse(struct peerline *, int, char *);

/* spp_ping */
void spp_cmd_ping(struct sppconn *);
void newpeer_ping(void *);
void handle_ping(struct sppconn *, struct req *);
void handleresp_ping(struct rep *, struct sppconn *);

/* spp_response */
void spp_cmd_rep(struct sppconn *, int);
void spp_cmd_rep2(struct sppconn *, struct rep *);

/* spp_revconn */
void spp_cmd_revconn(struct sppconn *, struct peer *);
void handle_revconn(struct sppconn *, struct req *);
void handleresp_revconn(struct rep *, struct sppconn *);
void revconn_file(void *);

/* spp_starttls */
void spp_cmd_starttls(struct sppconn *);
int handle_starttls(struct sppconn *);
void handleresp_starttls(struct rep *, struct sppconn *);

#endif /* _SPP_H_ */
