/* spp_get.c - spp GET command handling */

/*
 * Copyright (c) 2007, 2008, 2009, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: spp_get.c,v 1.63 2010/10/14 15:00:11 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "p2pd.h"
#include "dlagent.h"
#include "filemeta.h"
#include "peer.h"
#include "request.h"
#include "stream.h"
#include "spp.h"
#include "urient.h"

extern int extstreams;


/* spp_cmd_get - construct get request */
int
spp_cmd_get(struct sppconn *sc, off_t range_s, off_t range_e)
{
	const char *f = "spp_cmd_get";
	struct connection *c;
	struct peer *peer;
	char str[100];
	float rtt;
	char *enc;

	ASSERT(sc->dl != NULL, "sppconn without dl");

	peer = sc->conn->peer;
	c = sc->conn;

	ASSERT(c->obuflen == 0, "buffered data");
	ASSERT(c->ibuflen == 0, "buffered data");

	dbg("%s: %s %lld %lld %s", f, sc->ue->uri.uri,
	     (long long)range_s, (long long)range_e, peer->id);

	strlcpy(c->obuf, COMMAND_GET, sizeof(c->obuf));
	if ((enc = uri_encode(sc->ue->uri.path)) == NULL)
		return -1;
	strlcat(c->obuf, enc, sizeof(c->obuf));
	strlcat(c->obuf, " ", sizeof(c->obuf));
	strlcat(c->obuf, VALUE_VERSION_SPP, sizeof(c->obuf));
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));

	strlcat(c->obuf, OPTION_HOST, sizeof(c->obuf));
	if ((enc = uri_encode(sc->ue->uri.host)) == NULL)
		return -1;
	strlcat(c->obuf, enc, sizeof(c->obuf));
	strlcat(c->obuf, ":", sizeof(c->obuf));
	strlcat(c->obuf, sc->ue->uri.port, sizeof(c->obuf));
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));

	strlcat(c->obuf, OPTION_BINDPORT, sizeof(c->obuf));
	strlcat(c->obuf, lport, sizeof(c->obuf));
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));

	/* return RTT in ms/10 (XXXonly to server? need symbolic vs. ip test) */
	if (peerpar_getfloat2(c->peer, PEERPAR_LINK_RTT, &rtt) != -1) {
		strlcat(c->obuf, OPTION_RTT, sizeof(c->obuf));
		snprintf(str, sizeof(str), "%d", (int)(rtt * 10));
		strlcat(c->obuf, str, sizeof(c->obuf));
		strlcat(c->obuf, "\r\n", sizeof(c->obuf));
	}

	/* XXX use block range if whole block requested? */
	strlcat(c->obuf, OPTION_RANGE, sizeof(c->obuf));
	snprintf(str, sizeof(str), "bytes=%lld", (long long)range_s);
	strlcat(c->obuf, str, sizeof(c->obuf));
	/*XXX requests should always end on block boundary */
	if (range_e > 0) {
		snprintf(str, sizeof(str), "-%lld", (long long)range_e);
		strlcat(c->obuf, str, sizeof(c->obuf));
	}
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));

	if (strlcat(c->obuf, "\r\n", sizeof(c->obuf)) >= sizeof(c->obuf))
		ERRX("request too long");

	peerpar_settime(peer, PEERPAR_SPP_LASTREQ);

	c->obuflen = strlen(c->obuf);

	return 0;
}


/* handleresp_get - handle response to get request */
void
handleresp_get(struct rep *rep, struct sppconn *sc)
{
	const char *f = "handleresp_get";
	struct connection *c;
	int32_t bnum;

	c = sc->conn;
	bnum = sc->dlblock;

	/* XXX call urient_urifail() on e.g. 404 for disappearing files? */
	if (rep->code != 200) {
		dbg("%s: GET failure: %d for block %d from %s", f, rep->code,
		    bnum, c->peer->id);
		peerpar_adjint(c->peer, PEERPAR_SPP_FAILEDGET, 1);
		if (rep->code == 503)
			peerpar_setint(c->peer, PEERPAR_SPP_LASTRES,
				       PEER_PREVBUSY);
		else
			peerpar_setint(c->peer, PEERPAR_SPP_LASTRES,
				       PEER_PREVFAIL);

		dlagent_cmdfail(sc->dl, sc);
		conn_changestate(sc->conn, CONNECTION_STATUS_ENDED);
		return;
	}

	if (rep->bcnt > 0) {
		dbg("%s: adding %d subsequent block after block %d for peer %s",
		    f, rep->bcnt, bnum, c->peer->id);

		filemeta_peerupd(sc->ue->meta, c->peer, bnum + 1, rep->bcnt - 1,
				 BLOCKINFO_GET);
	}

/*	dbg("%s: GET %s block %d from %s", f, sc->ue->uri.path, bnum,
	     c->peer->id);*/
	sc->cachereq = 1;
	peerpar_adjint(c->peer, PEERPAR_SPP_OKCONN, 1);
	peerpar_setint(c->peer, PEERPAR_SPP_LASTRES, PEER_PREVOK);

	dlagent_cmdok(sc->dl, sc);
}


/* handle_get - process request for data range */
int
handle_get(struct sppconn *sc, struct req *req)
{
	const char *f = "handle_get";
	struct connection *c;
	struct filemeta *fm;
	time_t now, tdiff;
	struct rep rep;
	int32_t bnum;

	ASSERT(sc->ue != NULL, "NULL ue");

	c = SCTOC(sc);
	fm = sc->ue->meta;

	/*
	 * clients might use get without knowing file length, and for
	 * a cache the length will only become known after HEAD
	 * returns; set endoff to end of file here if not specified in
	 * request (streamreq should have already been denied if this
	 * should not be allowed).
	 */
	if (req->withend == 0)
		req->offend = fm->size - 1;

	sc->dlstart = req->offset;
	sc->dlend = req->offend;

	/* calculate block number for request */
	bnum = off2block(sc->ue, sc->dlstart);

	if (bnum >= sc->ue->meta->blockcnt) {
		swarn("%s: request for invalid block", f);
		spp_cmd_rep(sc, 403);
		return -1;
	}

	dbg("%s: %s block %d from %s (%lld -> %lld) %p", f,
	    sc->ue->uri.uri, bnum, c->peer->id, (long long)sc->dlstart,
	    (long long)sc->dlend, sc);

	/* XXX scc resource management */
	if (spp_extclient(sc)) {
		/* got request, postpone global revconn */
		revconn_timerupd();

		if (peerpar_getint(c->peer, PEERPAR_PEER_TOTCONN) >
		    PEER_CONN_MAX) {
			swarn("%s: too many connections from %s, denying", f,
			     c->peer->id);
			spp_cmd_rep(sc, 503);
			return -1;
		}

#if 0
		if (active_streams >= STREAM_ACTIVE_MAX) {
			dbg("%s: too many active streams, denying", f);
			spp_cmd_rep(c, 503);
			return -1;
		}
#endif

		if ((serv_mode && extstreams > MAXSTREAMS_SERVER) ||
		    (scc_mode && extstreams > MAXSTREAMS_SCC) ||
		    (extstreams > MAXSTREAMS_CLIENT)) {
			dbg("%s: too many active streams, denying %s", f,
			    c->peer->id);
			spp_cmd_rep(sc, 503);
			return -1;
		}
	}

#if 0
	{
		int optval;
		optval = 1024;
		if (setsockopt(c->fd, SOL_SOCKET, SO_SNDLOWAT, &optval, sizeof(optval)) == -1)
			swarn("%s: setsockopt: %m", f);

	}
#endif

	if (PTOSP(c->peer)->uncredited >
	    sc->ue->meta->blocksize * FILE_CREDIT_CHECKLIM)
		sppfile_wantcred(sc->ue);

	/* check that server recently contacted before denying */
	if (PTOSP(c->peer)->uncredited >
	    sc->ue->meta->blocksize * FILE_CREDIT_MAXLIM) {
		now = time(NULL);
		tdiff = now - UTOSPP(sc->ue)->lastcredres;

		if ((tdiff > 0) && (tdiff < FILE_CREDIT_FRESHLIM)) {
			dbg("%s: too much uncredited data from %s, denying", f,
			    c->peer->id);
			spp_cmd_rep(sc, 466); /* XXX Credit Limit Passed */
			return -1;
		} else
			dbg("%s: credit data too old, not blocking leecher", f);
	}

#if 0 /* XXX don't block leechers for now */
	if (peerpar_getint(c->peer, PEERPAR_SPP_LEECHER) == 1) {
		dbg("%s: request from leecher %s, denying", f, c->peer->id);
		spp_cmd_rep(sc, 467); /* XXX Leechers Denied */
		return -1;
	}
#endif

	/*XXX where/when to best verify end (will not know before head rep) */
	if (sc->dlend != 0 && sc->dlend >= fm->size) {
		dbg("%s: invalid range end value", f);
		spp_cmd_rep(sc, 503); /* XXX invalid offend */
		return -1;
	}

#if 0
	/* XXX */
	if ((lhc_mode || scc_mode) && sc->ue->cf == NULL) {
		if (urient_cachereq(sc->ue) == -1)
			swarn("%s: file might not be written to disk", f);
	}
#endif

#if 0
	/*
	 * XXX do this here? enables error return to client.
	 *     might be postponed if downloads without going to disk
	 *     are accepted
	 */
	if (urient_fileref(ue) == -1) {
		swarn("%s: urient_fileref failure", f);
		spp_cmd_rep(sc, 403);
		return -1;
	}
#endif

	bzero(&rep, sizeof(rep));
	/* return end offset of file to stream clients */
	if (sc->streamreq == 0) {
		/* return length of request */
		rep.code = 200;
		rep.size = (off_t)sc->dlend - sc->dlstart + 1;
		spp_cmd_rep2(sc, &rep);
	}
	else {
		rep.size = fm->size - sc->dlstart;
		if (serv_mode || req->http)
			rep.bcnt = 0;
		else
			/* how many extra blocks do we have? */
			rep.bcnt = urient_subseq(sc->ue, bnum);

		/* adjust content-length with regards to offset */
		if (req->flags & REQ_FLAG_PROXYRES)
			rep.code = 206;
		else
			rep.code = 200;
		spp_cmd_rep2(sc, &rep);
	}

	return 0;
}


/* sppget_postrep - after response has been written, start stream */
void
sppget_postrep(struct sppconn *sc)
{
	const char *f = "sppget_postrep";
	enum streamdeadline streamtype;
	struct connection *c;
	off_t reqlen;
	off_t reqend;

	c = sc->conn;

	dbg("%s: streaming %s from %s:%s starting at %lld", f,
	     sc->ue->uri.path, sc->ue->uri.host, sc->ue->uri.port,
	     (long long)sc->dlstart);

	/*
	 * use direction to determine type, XXX scc/server might wish to
	 * use deadlines, but more permissive than clients.
	 */
	if (c->type == CONN_TYPE_OUTGOING)
		streamtype = STREAM_DEADLINE;
	else
		streamtype = STREAM_NODEADLINE;
	stream_sendstart(streamtype, c, sc->ue, sc->dlstart, sc->dlend);

	if (sc->streamreq)
		UTOSPP(sc->ue)->laststreamreq = time(NULL);

	/* XXX have minimum request size? */
	if (sc->streamreq && sc->dlend == 0) {
		reqlen = sc->ue->meta->size - sc->dlstart;
		reqend = sc->ue->meta->size - 1;
	} else {
		reqlen = sc->dlend - sc->dlstart + 1;
		reqend = sc->dlend;
	}
	if (stream_availbytes(c->stream) < reqlen) {
		sc->dl = dlagent_search(sc->ue, sc->dlstart, reqend);
		/* XXX could try to send what we have */
		if (sc->dl == NULL) {
			conn_changestate(c, CONNECTION_STATUS_ENDED);
			return;
		}
		dlagent_addclient(sc->dl, sc);
	}
}
