/* spp_peers.c - PEERS spp command handling */

/*
 * Copyright (c) 2007, 2008, 2009, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: spp_peers.c,v 1.57 2010/07/31 09:48:09 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <string.h>

#include "p2pd.h"
#include "block.h"
#include "conn.h"
#include "dlagent.h"
#include "filemeta.h"
#include "peer.h"
#include "peerlist.h"
#include "peerset.h"
#include "request.h"
#include "spp.h"
#include "urient.h"


/* spp_cmd_peers - construct peers request to parent node */
int
spp_cmd_peers(struct sppconn *sc, int32_t bnum)
{
	const char *f = "spp_cmd_peers";
	off_t range_s, range_e;
	struct connection *c;
	char str[100];
	char *enc;

	dbg("%s: %s (%d)", f, sc->ue->uri.uri, bnum);

#if 0 /* XXXends up with one per request */
	/*
	 * send unreported igot messages at the same time,
	 * want to do this here in order to build up priority
	 */
	igot_flush();
#endif

	c = sc->conn;

	/*XXX*/
	range_s = block2off(sc->ue, bnum);
	range_e = range_s + sc->ue->meta->blocksize - 1;

	ASSERT(c->obuflen == 0, "buffered data");

	strlcpy(c->obuf, COMMAND_PEERS, sizeof(c->obuf));
	if ((enc = uri_encode(sc->ue->uri.path)) == NULL)
		return -1;
	strlcat(c->obuf, enc, sizeof(c->obuf));
	strlcat(c->obuf, " ", sizeof(c->obuf));
	strlcat(c->obuf, VALUE_VERSION_SPP, sizeof(c->obuf));
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));

	strlcat(c->obuf, OPTION_HOST, sizeof(c->obuf));
	if ((enc = uri_encode(sc->ue->uri.host)) == NULL)
		return -1;
	strlcat(c->obuf, enc, sizeof(c->obuf));
	strlcat(c->obuf, ":", sizeof(c->obuf));
	strlcat(c->obuf, sc->ue->uri.port, sizeof(c->obuf));
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));

	strlcat(c->obuf, OPTION_BINDPORT, sizeof(c->obuf));
	strlcat(c->obuf, lport, sizeof(c->obuf));
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));

	/* XXX PARAM_BLOCKRANGE "blocks=a-b" | "blocks=a" */
	strlcat(c->obuf, OPTION_RANGE, sizeof(c->obuf));
	strlcat(c->obuf, PARAM_BYTERANGE, sizeof(c->obuf));
	snprintf(str, sizeof(str), "%lld", (long long)range_s);
	strlcat(c->obuf, str, sizeof(c->obuf));
	/*XXX requests should always end on block boundary */
	if (range_e > 0) {
		snprintf(str, sizeof(str), "-%lld", (long long)range_e);
		strlcat(c->obuf, str, sizeof(c->obuf));
	}
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));

	/* return RTT in ms/10 */
	if (peerpar_getfloat2(c->peer, PEERPAR_LINK_RTT, NULL) != -1) {
		strlcat(c->obuf, OPTION_RTT, sizeof(c->obuf));
		snprintf(str, sizeof(str), "%d",
			 (int)(peerpar_getfloat(c->peer, PEERPAR_LINK_RTT) * 10));
		strlcat(c->obuf, str, sizeof(c->obuf));
		strlcat(c->obuf, "\r\n", sizeof(c->obuf));
	}

	if (strlcat(c->obuf, "\r\n", sizeof(c->obuf)) >= sizeof(c->obuf))
		ERRX("request too long");

/*	dbg("%s: %s", f, c->obuf);*/

	c->obuflen = strlen(c->obuf);

	return 0;
}


/* handleresp_peers - handle response to peers request */
void
handleresp_peers(struct rep *rep, struct sppconn *sc)
{
	const char *f = "handleresp_peers";
	size_t maxlen;

	dbg("%s", f);

	/*XXX should have whoami command */
	if (strlen(mysockname) == 0) {
		maxlen = sizeof(mysockname);
		if (strlcpy(mysockname, sc->conn->sockname, maxlen) >= maxlen)
			ERRX("mysocklen too long");
	}

	if (rep->code != 200) {
		/*
		 * XXX could happen if file deleted from server,
		 *     need to handle this in special way
		 */
		swarn("%s: PEERS failed: %d", f, rep->code);
		errno_spp = rep->code;
		dlagent_cmdfail(sc->dl, sc);
		cache_connection(sc->conn);
		return;
	}

	/*XXX probably needed for other commands also */
	if (rep->size == 0) {
		/* not much to do in this case */
		swarn("%s: successful PEERS with no content", f);
		errno_spp = 403; /*XXX resp code? */
		dlagent_cmdfail(sc->dl, sc);
		cache_connection(sc->conn);
		return;
	}

	/* prepare to read peerlist */
	sc->conn->ibufexplen = rep->size;
	conn_changestate(sc->conn, CONNECTION_STATUS_READREPDAT);
}


/* parse_peerlist - process peerlist data */
void
parse_peerlist(struct sppconn *sc)
{
	const char *f = "parse_peerlist";
	struct peerline plist[MAX_REDIRECT];
	struct connection *c;
	struct dlagent *dl;
	struct speer *sp;
	int32_t bnum;
	struct peer *peer;
	int pcnt;
	int i;

	ASSERT(sc->dl != NULL, "NULL dl");

/*	dbg("%s", f);*/

	c = sc->conn;
	if ((pcnt = peers_parse(&plist[0], MAX_REDIRECT, c->ibuf)) == -1) {
		swarn("%s: invalid PEERS reply, ending connection", f);
		conn_changestate(c, CONNECTION_STATUS_ENDED);
		c->ibuflen = 0;
		return;
	}
	c->ibuflen = 0;

	ASSERT(pcnt <= MAX_REDIRECT, "pcnt value too large");
	bnum = sc->dlblock; /*XXX*/
	for (i = 0; i < pcnt; i++) {
		if (((strcmp(plist[i].addr, c->sockname) == 0)) &&
		    (strcmp(plist[i].port, lport) == 0)) {
			dbg("%s: redirect to self, ignoring", f);
			continue;
		}
		if ((sp = speer_add(plist[i].addr, plist[i].port)) == NULL)
			ERRX("speer_add failed");
		peer = SPTOP(sp);
		dbg("%s: %s servrank: %d %s", f, sc->ue->uri.uri, i, peer->id);
		if (bnum + plist[i].cnt >= sc->ue->meta->blockcnt) {
			dbg("%s: invalid blockcount %d for host %s:%s, block %d",
			    f, plist[i].cnt, peer->name, peer->port, bnum);
			ERRX("invalid blockcnt");
			/* continue; */
		}
		filemeta_peerupd(sc->ue->meta, peer, bnum, plist[i].cnt, BLOCKINFO_PEERS);
		/* server score, based on ranking done by server */
		peerpar_setint(peer, PEERPAR_SPP_SSCORE, i);
		peerpar_setint(peer, PEERPAR_SPP_PREVBCNT, plist[i].cnt);
		speer_wantinfo(sc->ue, sp);
	}

	dbg("%s: peer request ended, caching conn", f);

	/* cache before cmdok, which starts GET */
	dl = sc->dl; /* cleared by cache_connection */
	cache_connection(c);
	dlagent_cmdok(dl, sc);
}


/* handle_peers - process peers request (server/scc) */
void
handle_peers(struct sppconn *sc, struct req *req)
{
	const char *f = "handle_peers";
	struct peerline pl[MAX_REDIRECT];
	char redir[4096]; /*XXX*/
	struct connection *c;
	struct peerset *pset;
	struct peer *peer;
	char str[1024];
	int32_t mincnt;
	int32_t bnum;
	int i, j;

	(void)req;

	c = sc->conn;

	ASSERT(c->obuflen == 0, "buffered data");

	/* calculate block number for request */
	bnum = off2block(sc->ue, sc->dlstart);
	/*XXX store peer info to disk? */
	dbg("%s: PEERS request for %s %d from %s", f, sc->ue->uri.uri, bnum,
	    c->peer->id);

	pset = peerlist_gen(sc->ue, bnum);
	/* only return local clients in scc mode */
	if (scc_mode && pset != NULL)
		for (i = pset->peercnt; i > 0; i--)
			if (PTOSP(pset_peer(pset, i - 1))->sccclient == 0)
				pset_delpeer(pset, pset_peer(pset, i - 1));

	/* could remove client, but not likely to ask if it already has block */

	j = 0;
	mincnt = 0;
	if ((pset != NULL) && (pset->peercnt > 0)) {
		/*
		 * sort peers based what we estimate to be the best suited
		 * peers first (with best essentially being the node with
		 * highest transfer rates possible, even though this is
		 * not something we can easily determine).
		 */
		/* XXX randomize as alternative? */
		peerlist_servsort(pset, c->peer);

		for (i = 0; i < MIN(pset->peercnt, MAX_REDIRECT - 1); i++) {
			peer = pset_peer(pset, i);
			pl[j].rttdist = peerpar_getint(peer, PEERPAR_TMP_INT1);
			strlcpy(pl[j].addr, peer->name, sizeof(pl[j].addr));
			strlcpy(pl[j].port, peer->port, sizeof(pl[j].port));
			pl[j].cnt = filemeta_peerseq(sc->ue->meta, bnum, peer);
			if ((mincnt == 0) || (pl[j].cnt < mincnt))
				mincnt = pl[j].cnt;
			j++;
		}
	}
	i = j;

	/* try to reduce peers requests */
	if (mincnt == 0)
		/* XXX set to server/scc 'load' value? */
		mincnt = MIN(5, sc->ue->meta->blockcnt - 1 - bnum);

#ifdef TESTFAIL
	/*XXX skip server if possible to aid testing */
	if (pset == NULL || pset->peercnt == 0 || j == 0) {
#endif /* TESTFAIL */
		/* put server/scc in last */
		pl[i].rttdist = peerpar_getfloat(c->peer, PEERPAR_LINK_RTT);
		strlcpy(pl[i].addr, c->sockname, sizeof(pl[i].addr));
		strlcpy(pl[i].port, c->sockport, sizeof(pl[i].port));
		pl[i].cnt = mincnt; /* same as lowest peer value */
		i++;
#ifdef TESTFAIL
	}
#endif /* TESTFAIL */

	/* format is a newline separated "ip port cnt" list */
	bzero(redir, sizeof(redir));
	for (j = 0; j < i; j++) {
		dbg(" -> %s:%s", pl[j].addr, pl[j].port);
		strlcat(redir, pl[j].addr, sizeof(redir));
		strlcat(redir, " ", sizeof(redir));
		strlcat(redir, pl[j].port, sizeof(redir));
		strlcat(redir, " ", sizeof(redir));
		if (pl[j].cnt + bnum >= sc->ue->meta->blockcnt)
			ERRX("cnt too large");
		snprintf(str, sizeof(str), "%d", pl[j].cnt);
		strlcat(redir, str, sizeof(redir));
		if (strlcat(redir, "\r\n", sizeof(redir)) >= sizeof(redir))
			ERRX("request too long");
	}

	strlcpy(c->obuf, "SPP/1.0 200 OK\r\n", sizeof(c->obuf));
	strlcat(c->obuf, "Content-Length: ", sizeof(c->obuf));
	snprintf(str, sizeof(str), "%d", (int)strlen(redir));
	strlcat(c->obuf, str, sizeof(c->obuf));
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));
	if (strlcat(c->obuf, redir, sizeof(c->obuf)) >= sizeof(c->obuf))
		ERRX("request too long");

	c->obuflen = strlen(c->obuf);
	conn_changestate(c, CONNECTION_STATUS_WRITEREP);

	if (pset != NULL)
		pset_free(pset);
}
