/* spp_response.c - spp command response building functions */

/*
 * Copyright (c) 2007, 2008, 2009, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: spp_response.c,v 1.27 2010/07/31 09:48:09 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <sys/types.h>
#include <sys/socket.h>

#include <stdio.h>
#include <string.h>

#include "p2pd.h"
#include "conn.h"
#include "peer.h"
#include "spp.h"


/* spp_cmd_rep - simplified spp_cmd_rep2 wrapper */
void
spp_cmd_rep(struct sppconn *sc, int code)
{
	struct rep rep;

	bzero(&rep, sizeof(rep));
	rep.code = code;
	spp_cmd_rep2(sc, &rep);
}


/* spp_cmd_rep2 - return response baaed on values in rep */
void
spp_cmd_rep2(struct sppconn *sc, struct rep *rep)
{
	const char *f = "spp_cmd_rep2";
	struct connection *c;
	char str[100];

/*	dbg("%s", f);*/

	c = sc->conn;
	/* XXX invalid, too long request triggers this */
	ASSERT(c->obuflen == 0, "buffered data");

	if (sc->httpreq) {
		strlcpy(c->obuf, "HTTP/1.0 ", sizeof(c->obuf));
		dbg("%s: sending HTTP reply, code %d", f, rep->code);
	} else
		strlcpy(c->obuf, "SPP/1.0 ", sizeof(c->obuf));

	snprintf(str, sizeof(str), "%d", rep->code);
	strlcat(c->obuf, str, sizeof(c->obuf));
	/* XXX message */
	if (rep->code == 200 || rep->code == 206)
		strlcat(c->obuf, " OK", sizeof(c->obuf));
	else
		strlcat(c->obuf, " Error", sizeof(c->obuf));
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));

	if (rep->size != 0) {
		if (sc->httpreq) {
			/* XXX mplayer doesn't like this value */
/*			strlcat(c->obuf,
				"Content-Type: application/octet-stream\r\n",
				sizeof(c->obuf));*/
			strlcat(c->obuf, "Accept-Ranges: bytes\r\n",
				sizeof(c->obuf));
		}
		strlcat(c->obuf, OPTION_CONTLEN, sizeof(c->obuf));
		snprintf(str, sizeof(str), "%lld", (long long)rep->size);
		strlcat(c->obuf, str, sizeof(c->obuf));
		strlcat(c->obuf, "\r\n", sizeof(c->obuf));
	}

	if (rep->bcnt != 0) {
		ASSERT(sc->httpreq == 0, "bcnt on HTTP reply");
		strlcat(c->obuf, OPTION_SUBSEQ, sizeof(c->obuf));
		snprintf(str, sizeof(str), "%d", rep->bcnt);
		strlcat(c->obuf, str, sizeof(c->obuf));
		strlcat(c->obuf, "\r\n", sizeof(c->obuf));
	}

	if (rep->nounce != 0) {
		ASSERT(sc->httpreq == 0, "nonuce in HTTP reply");
		strlcat(c->obuf, OPTION_NOUNCE, sizeof(c->obuf));
		snprintf(str, sizeof(str), "%d", rep->nounce);
		strlcat(c->obuf, str, sizeof(c->obuf));
		strlcat(c->obuf, "\r\n", sizeof(c->obuf));
	}

	if (recovery && (PTOSP(c->peer)->sentrecov == 0)) {
		strlcat(c->obuf, OPTION_DBREFRESH, sizeof(c->obuf));
		strlcat(c->obuf, "\r\n", sizeof(c->obuf));
		PTOSP(c->peer)->sentrecov = 1;
	}

	if (rep->blocksize != 0) {
		ASSERT(sc->httpreq == 0, "blocksize in HTTP reply");
		strlcat(c->obuf, OPTION_BLOCKSIZE, sizeof(c->obuf));
		snprintf(str, sizeof(str), "%u", rep->blocksize);
		strlcat(c->obuf, str, sizeof(c->obuf));
		strlcat(c->obuf, "\r\n", sizeof(c->obuf));
	}

	if (strlcat(c->obuf, "\r\n", sizeof(c->obuf)) >= sizeof(c->obuf))
		ERRX("reply too long");

	dbg("%s: spp response: %s", f, c->obuf);

	conn_changestate(c, CONNECTION_STATUS_WRITEREP);
	sc->sppcode = rep->code;

	c->obuflen = strlen(c->obuf);
}
