/* spp_starttls.c - spp starttls command handling */

/*
 * Copyright (c) 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: spp_starttls.c,v 1.17 2010/10/11 17:44:38 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <string.h>

#include "p2pd.h"
#include "conn.h"
#include "peer.h"
#include "spp.h"
#include "ssl.h"
#include "urient.h"


/* spp_cmd_starttls - request starting of ssl negotiation */
void
spp_cmd_starttls(struct sppconn *sc)
{
	const char *f = "spp_cmd_starttls";
	struct connection *c;
	char *enc;

	dbg("%s: %s", f, sc->ue->uri.uri);

	c = sc->conn;

	ASSERT(c->peer->sslactive == 0, "peer already sslactive");
	ASSERT(c->obuflen == 0, "buffered data");
	ASSERT(c->ibuflen == 0, "buffered data");

	strlcpy(c->obuf, COMMAND_STARTTLS, sizeof(c->obuf));
	/*
	 * XXX should path be used? having it for consistency would be
	 *     nice (and to allow for early abort if file for some
	 *     reason does not exist at server), however the
	 *     established context should be general (i.e., not
	 *     limited to reporting data for the same file).
	 */
	if ((enc = uri_encode(sc->ue->uri.path)) == NULL)
		return;
	strlcat(c->obuf, enc, sizeof(c->obuf));
	strlcat(c->obuf, " ", sizeof(c->obuf));
	strlcat(c->obuf, VALUE_VERSION_SPP, sizeof(c->obuf));
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));

	strlcat(c->obuf, OPTION_HOST, sizeof(c->obuf));
	if ((enc = uri_encode(sc->ue->uri.host)) == NULL)
		return;
	strlcat(c->obuf, enc, sizeof(c->obuf));
	strlcat(c->obuf, ":", sizeof(c->obuf));
	strlcat(c->obuf, sc->ue->uri.port, sizeof(c->obuf));
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));

	strlcat(c->obuf, OPTION_BINDPORT, sizeof(c->obuf));
	strlcat(c->obuf, lport, sizeof(c->obuf));
	strlcat(c->obuf, "\r\n", sizeof(c->obuf));

	if (strlcat(c->obuf, "\r\n", sizeof(c->obuf)) >= sizeof(c->obuf))
		ERRX("request too long");

	c->obuflen = strlen(c->obuf);
}


#if HAVE_OPENSSL


/* handle_starttls - handle starttls request */
int
handle_starttls(struct sppconn *sc)
{
	const char *f = "handle_starttls";

	if (use_ssl) {
		dbg("%s: starttls request from %s", f, sc->conn->peer->id);
		spp_cmd_rep(sc, 200);
	} else {
		dbg("%s: no ssl support", f);
		spp_cmd_rep(sc, 501);
	}

	return 0;
}


/* handleresp_starttls - handle response to starttls */
void
handleresp_starttls(struct rep *rep, struct sppconn *sc)
{
	const char *f = "handleresp_starttls";

	if (rep->code != 200) { /* XXX proceed without ssl? */
		ERRX("starttls response error");
		PTOSP(sc->sslinitpeer)->sslinitactive = 0;
		dbg("%s: sslinit failure for %s", f, sc->sslinitpeer->id);
		sc->sslinitpeer = NULL;
	}

	ssl_connect(sc->conn);
}

#else

#include "p2pd.h"

/* handle_starttls - handle starttls request */
int
handle_starttls(struct sppconn *sc)
{
	const char *f = "handle_starttls";

	dbg("%s: no ssl support", f);
	spp_cmd_rep(sc, 501);

	return 0;
}


/* handleresp_starttls - handle response to starttls */
void
handleresp_starttls(struct rep *rep, struct sppconn *sc)
{
	(void)rep;
	(void)sc;

	ERRX("CLIENT_CMD_STARTTLS");
}

#endif /* HAVE_OPENSSL */
