/* sppfile_cred.c - cred command related state handling for spp files */

/*
 * Copyright (c) 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: sppfile_cred.c,v 1.6 2010/06/29 15:14:38 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <time.h>

#include "p2pd.h"
#include "spp.h"
#include "urient.h"


/* sppfile_wantcred - want cred info from the server peer */
void
sppfile_wantcred(struct urient *ue)
{
	const char *f = "sppfile_wantcred";
	struct sppconn *sc;

	/*
	 * cred info not really urient specific, but used for
	 * consistency; need to include it in command.
	 */

	dbg("%s", f);

	/*XXX would like to have cred_ongoing value */

	/* do nothing if cred recently issued */
	if (UTOSPP(ue)->lastcredcheck != 0 &&
	    UTOSPP(ue)->lastcredcheck + SPP_CRED_MINDELAY < time(NULL)) {
		dbg("%s: previous cred check too recent", f);
		/*XXX set timer to try again or assume speer_wantcred called? */
		return;
	}

	sc = sppconn_new(CLIENT_CMD_CRED);
	sppconn_setfile(sc, ue);
	spp_parentreq(sc);
}

/* XXX would ideally have error handling, or generic retry handler for
 *     commands
 */
