/* alist.h - alist structures and prototypes */

/*
 * Copyright (c) 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: alist.h,v 1.12 2010/06/25 22:02:25 karls Exp $ */

#ifndef _ALIST_H_
#define _ALIST_H_

/* alist.h - array list defines and prototypes */
struct alent {
	void *data;
	void *key;
	time_t lru; /* for lru expired entries */
};

enum alsizing {
	ALIST_SIZE_FIXED,
	ALIST_SIZE_GROW,
	ALIST_SIZE_LRU,
	ALIST_SIZE_FIFO,
};

enum altype {
	ALIST_TYPE_PTR,
	ALIST_TYPE_STR,
	ALIST_TYPE_UNSORTED_STR,
	ALIST_TYPE_UNSORTED_PTR,
};

enum aldir {
	ALIST_SHIFT_LEFT,
	ALIST_SHIFT_RIGHT,
};

struct alist {
	void (*print)(const void *);
	void (*free)(void *);
	int (*sort)(const void *, const void *);
	struct alent *arr;
	size_t size;
	size_t arrsize;
	enum alsizing sizing;
	enum altype type; /* pointer/string */
};

/* XXX PVAL does not check for out-of-bounds dereference */
#define ALIST_PVAL(alist, i) ((alist)->arr[i].data)
#define ALIST_SIZE(alist) ((alist)->size)

struct alist *alist_init(void (*)(const void *),
			 void (*)(void *), size_t, enum alsizing, enum altype);
void *alist_add(struct alist *, void *, void *);
void alist_delidx(struct alist *, size_t);
void alist_del(struct alist *, void *);
void *alist_find(struct alist *, void *);
void *alist_rfind(struct alist *, void *, size_t);
void alist_shift(struct alist *, size_t, size_t, int);
void alist_trim(struct alist *, size_t);
void alist_mysort(struct alist *, size_t, int (*)(const void *, const void *));
void alist_free(struct alist *);
void alist_dump(struct alist *);

#endif /* !_ALIST_H_ */
