/* bitfield.c - encoding and decoding functions for bit strings */

/*
 * Copyright (c) 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: bitfield.c,v 1.5 2010/06/25 22:02:25 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <string.h>

#include "log.h"
#include "bitfield.h"
#include "blockmap.h"

/* XXX current encoding approach is very inefficient */

#define BITFIELD_ENCSTR_MAX 8192 /*XXX max block count */


/* bitfield_encstr - encode bit string as string */
char *
bitfield_encstr(struct blockmap *bm)
{
	static char str[BITFIELD_ENCSTR_MAX];
	int cnt;
	int i;

	ASSERT(bm->size > 0, "invalid fieldlen");

	/* any bits set? */
	cnt = 0;
	for (i = 0; i < bm->size; i++) {
		if (blockmap_test(bm, i))
			cnt++;
	}

#if 0 /*XXX this type of thing should be handled by caller */
	/* return NULL for empty fields */
	if (cnt == 0)
		return NULL;
#endif

	/* build string */
	for (i = 0; i < bm->size; i++) {
		if (blockmap_test(bm, i))
			str[i] = '1';
		else
			str[i] = '0';
	}
	str[bm->size] = '\0';

	return str;
}


/* bitfield_decstr - decode string with bitfield data, return -1 on failure */
int
bitfield_decstr(struct blockmap *bm, char *str)
{
	const char *f = "bitstr_decstr";
	int i;

	/* verify length and contents of bit string */
	if (strlen(str) != (size_t)bm->size) {
		swarn("%s: incorrect bitstring length", f);
		return -1;
	}
	if (strlen(str) != strspn(str, "10")) {
		swarn("%s: invalid bitstring", f);
		return -1;
	}

	/* set bits in block map */
	for (i = 0; i < bm->size; i++)
		if (str[i] == '1')
			blockmap_set(bm, i);

	return 0;
}
