/* blockmap.h - block map portability handling, prototypes and structures */

/*
 * Copyright (c) 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: blockmap.h,v 1.8 2010/06/25 22:02:26 karls Exp $ */

#ifndef _BLOCKMAP_H_
#define _BLOCKMAP_H_

#if HAVE_BITSTRING_H
#include <bitstring.h>
#else
#include "bitstring.h"
#endif /* HAVE_BITSTRING_H */

struct blockmap {
	int32_t size;
	bitstr_t *map;
};

void blockmap_init(struct blockmap *, int32_t);
void blockmap_set(struct blockmap *, int32_t);
void blockmap_clear(struct blockmap *, int32_t);
void blockmap_nclear(struct blockmap *, int32_t, int32_t);
int blockmap_test(const struct blockmap *, int32_t);
void blockmap_free(struct blockmap *);
int32_t blockmap_blockcount(struct blockmap *, int32_t);
int32_t blockmap_subseq(struct blockmap *, int32_t);
int blockmap_iscomp(struct blockmap *);

#endif /* _BLOCKMAP_H_ */
