/* callback.c - interface for having callbacks for protocol handlers */

/*
 * Copyright (c) 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: callback.c,v 1.5 2010/06/25 22:02:26 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <string.h>

#include "log.h"
#include "alist.h"
#include "callback.h"

#define CALLBACK_MAX 20

struct alist *al_callback;

/* XXX use same approach as peer.h? protocol enum (set at compile time)*/

/* callback_preinit - setup callback list */
void
callback_preinit(void)
{
	/* use fixed size array, a huge number likely not needed */
	al_callback = alist_init((void (*)(const void *))callback_print,
				 NULL, CALLBACK_MAX, ALIST_SIZE_FIXED,
				 ALIST_TYPE_STR);
}


/* callback_add - add new callback with specified protocol handlers */
void
callback_add(char *protname,
	     void (*got_req)(void *),
	     void *(*new_urient)(void *))
{
	struct callback *cb;
	size_t maxlen;

	ASSERT(alist_find(al_callback, protname) == NULL, "duplicate callback");

	cb = xmalloc(sizeof(struct callback));
	cb->got_req = got_req;
	cb->new_urient = new_urient;
	maxlen = sizeof(cb->protname);
	if (strlcpy(cb->protname, protname, maxlen) >= maxlen)
		ERRX("protname too long");

	if (alist_add(al_callback, protname, cb) == NULL)
		ERRX("unable to add new callback");
}


/* callback_find - lookup callback for given protname */
struct callback *
callback_find(char *protname)
{
	return alist_find(al_callback, protname);
}

/* callback_print - protocol names */
void
callback_print(const struct callback *cb)
{
	dbg(" '%s'", cb->protname);
}


/* callback_dump - dump all supported  */
void
callback_dump(void)
{
	const char *f = "callback_dump";

	dbg("%s: enabled protocols:", f);
	alist_dump(al_callback);
}
