/* digest.h - wrapper around openssl/native digest functions */

/*
 * Copyright (c) 2007, 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: digest.h,v 1.14 2010/06/30 08:37:59 karls Exp $ */

#ifndef _DIGEST_H_
#define _DIGEST_H_

#if !DIGEST_OPENSSL

#if HAVE_MD5INIT && HAVE_RMD160INIT
#include <md5.h>
#include <rmd160.h>
#else
#include "md5.h"
#include "rmd160.h"
#endif /* HAVE_MD5INIT && HAVE_RMD160INIT */

#else /* DIGEST_OPENSSL */

#include <openssl/evp.h>

#endif /* DIGEST_OPENSSL */

/*
 * Framework for handling different message digest implementations.
 * Digests can either be calculated with a native digest (currently
 * rmd160 and md5), or via OpenSSL.
 */

#define DIGEST_NAMELEN 10 /* max length of name, includes NUL */

#define DIGEST_MD5 "md5"
#define DIGEST_RMD160 "rmd160"

/* make MD5 default digest, not too cpu intensive */
#define DIGEST_DEFAULT DIGEST_MD5

#if !DIGEST_OPENSSL
#define DIGEST_MAXLEN MAX(MD5_DIGEST_STRING_LENGTH, RMD160_DIGEST_STRING_LENGTH)
#else /* DIGEST_OPENSSL */
#define DIGEST_MAXLEN EVP_MAX_MD_SIZE
#endif /* DIGEST_OPENSSL */

enum native_md_t { md_md5, md_rmd160 };

struct digest {
#if !DIGEST_OPENSSL
	enum native_md_t type;
	MD5_CTX *md5ctx;
	RMD160_CTX *rmd160ctx;
#else /* DIGEST_OPENSSL */
	EVP_MD_CTX *evpctx;
	int len;
#endif /* DIGEST_OPENSSL */
};

extern char digest_default[];

void digest_preinit(char *);
struct digest *digest_init(char *);
int digest_validtype(char *);
int digest_mdlen(char *);
void digest_update(struct digest *, const char *, size_t);
char *digest_end(struct digest *);
void digest_free(struct digest *);

#endif /* _DIGEST_H_ */
