/* diskcache.h - diskcache defines and prototypes */

/*
 * Copyright (c) 2008, 2009, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: diskcache.h,v 1.10 2010/06/30 08:37:59 karls Exp $ */

#ifndef DISKCACHE_H_
#define DISKCACHE_H_

struct diskcache {
	off_t used;   /* currently stored data */
	off_t max;    /* upper limit on cache size */
	off_t commit; /* 'committed' storage size, based on file size */
};

#define CACHEIDX_NAME "cacheidx.txt"
#define CACHEIDX_HDR "CDBV0"

/*XXX verify maxlength with filemeta/cachefile data encoding lengths */
#define CACHEIDX_MAXLINE 8192

struct urient;

/* diskcache.c */
void diskcache_preinit(unsigned int, char *);
int diskcache_newfile(struct urient *);
void diskcache_expire(void);
void diskcache_delfile(struct urient *);
void diskcache_clean(void);
off_t diskcache_sizeleft(void);
void diskcache_gotblock(struct urient *, int32_t);
void diskcache_lostblock(struct urient *, int32_t);
void diskcache_loadidx(void);
void diskcache_saveidx(void);
void diskcache_status(int);

/* internal */
void diskcache_print(const void *);
void diskcache_free(void *);

#endif /* DISKCACHE_H_ */
