/* filemeta.h - filemeta structures and function prototypes */

/*
 * Copyright (c) 2008, 2009, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: filemeta.h,v 1.20 2010/06/27 14:18:03 karls Exp $ */

#ifndef _FILEMETA_H_
#define _FILEMETA_H_

#include "blockmap.h"
#include "digest.h"
#include "peer.h"

struct fm_peer {
	struct peer *peer;
	struct blockmap bm;
};

struct filemeta {
	struct uri *uri;
	off_t size;
	int32_t blockcnt; /* number of blocks in file (~size/blocksize) */
	uint32_t blocksize;
	struct alist *al_peers; /* list of peers with file */
	char digest[DIGEST_NAMELEN]; /* file digest type */
};

/* filemeta.c */
int filemeta_addserv(char *, char *);
struct filemeta *filemeta_new(struct uri *, off_t, int, char *);
void filemeta_free(struct filemeta *);
void filemeta_peerupd(struct filemeta *, struct peer *, int32_t, int32_t,
		      char *);
int32_t filemeta_peerseq(struct filemeta *, int32_t, struct peer *);
void filemeta_delpeer(struct filemeta *, struct peer *);
void filemeta_dump(struct filemeta *);
int32_t filemeta_size2blockcnt(off_t, int);
struct filemeta *filemeta_decstr(struct uri *, char *);
char *filemeta_encstr(struct filemeta *);

/* internal */
struct fm_peer *filemeta_newpeer(struct peer *, int);
void filemeta_freepeer(void *);
void filemeta_printpeer(const struct fm_peer *);

#endif /* _FILEMETA_H_ */
