/* iostat.c - net and disk io statistics + resource usage */

/*
 * Copyright (c) 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: iostat.c,v 1.10 2010/06/29 14:35:59 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <sys/time.h>
#include <sys/resource.h>

#include "log.h"
#include "iostat.h"

struct iostats ios;


/* iostat_dump - show getrusage() and io statistics */
void
iostat_dump(void)
{
	struct rusage ru;

	if (getrusage(RUSAGE_SELF, &ru) == -1)
		ERR("getrusage");

	dbg("utime: %ld.%ld stime: %ld.%ld maxrss: %ld ixrss: %ld idrss: %ld "
	    "isrss: %ld minflt: %ld majflt: %ld nswap: %ld inblock: %ld "
	    "oublock: %ld nivcsw: %ld nivcsw: %ld", (long)ru.ru_utime.tv_sec,
	    (long)ru.ru_utime.tv_usec, (long)ru.ru_stime.tv_sec,
	    (long)ru.ru_stime.tv_usec, ru.ru_maxrss, ru.ru_ixrss, ru.ru_idrss,
	    ru.ru_isrss, ru.ru_minflt, ru.ru_majflt, ru.ru_nswap, ru.ru_inblock,
	    ru.ru_oublock, ru.ru_nvcsw, ru.ru_nivcsw);

	dbg("iostats: disk_wdat: %lld disk_rdat: %lld disk_woh: %lld "
	    "disk_roh: %lld net_wdat: %lld net_rdat: %lld net_woh: %lld "
	    "net_roh: %lld net_wdat_s: %lld", (long long)ios.disk_wdat,
	    (long long)ios.disk_rdat, (long long)ios.disk_woh,
	    (long long)ios.disk_roh, (long long)ios.net_wdat,
	    (long long)ios.net_rdat, (long long)ios.net_woh,
	    (long long)ios.net_roh, (long long)ios.net_wdat_s);
}
