/* log.h - logging macros and functions, inclusion of platform headers */

/*
 * Copyright (c) 2007, 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: log.h,v 1.26 2010/07/17 18:07:23 karls Exp $ */

#include <sys/time.h>

#include <limits.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <syslog.h>

#if HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

/* log.h is assumed to be included by all other files, include osdep/config */
#include "osdep.h"
#include "config.h"

#define FILESIZE_MAX LLONG_MAX /* XXX off_t */

extern int debug;
extern int logsyslog;
extern int logdatepid;
extern int abortonerr;
extern int logerrline;
extern int extradebug;

/* log */
#undef ERR /*XXXsunos */
#define ERR(msg) do \
{ \
	if (logerrline) \
		slog(LOG_ERR, "internal error: %s:%d: %s: %m", __FILE__, __LINE__, (msg)); \
	else \
		slog(LOG_ERR, "internal error: %s: %s: %m", __FILE__, (msg)); \
	if (abortonerr) \
		abort(); \
	else \
		exit(1); \
	/*NOTREACHED*/ \
} while (/* CONSTCOND */ 0);

#define ERRX(msg) do \
{ \
	if (logerrline) \
		slog(LOG_ERR, "internal error: %s:%d: %s", __FILE__, __LINE__, (msg)); \
	else \
		slog(LOG_ERR, "internal error: %s: %s", __FILE__, (msg)); \
	if (abortonerr) \
		abort(); \
	else \
		exit(1); \
	/*NOTREACHED*/ \
} while (/* CONSTCOND */ 0);

#define ASSERT(cond, msg) do \
{ \
	if (!(cond)) \
		ERRX(msg); \
} while (/* CONSTCOND */ 0);

enum logtype {
	log_syslog,
	log_stdout,
	log_stderr,
	log_file,
};

int log_preinit(char *, enum logtype, char *);

void serr(const char *, ...)
	__attribute__((__format__ (printf, 1, 2)))
	__attribute__((noreturn));
void dbg(const char *, ...) __attribute__((__format__ (printf, 1, 2)));
void dbg2(const char *, ...) __attribute__((__format__ (printf, 1, 2)));
void swarn(const char *, ...) __attribute__((__format__ (printf, 1, 2)));
void sinfo(const char *, ...) __attribute__((__format__ (printf, 1, 2)));
void slog(int, const char *, ...) __attribute__((__format__ (printf, 2, 3)));

/* XXX move to mem/wrap.c */
void *xmalloc(size_t);
void *xcalloc(size_t, size_t);
void *xrealloc(void *, size_t);
char *xstrdup(const char *);

int xgettimeofday(struct timeval *, struct timezone *);
void xclose(int);
