/* peerpar.h - peer parameter defines, structures and prototypes */

/*
 * Copyright (c) 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: peerpar.h,v 1.6 2010/06/25 22:02:28 karls Exp $ */
#ifndef _PEERPAR_H_
#define _PEERPAR_H_

#include "peerpar_parvals.h"

#ifndef PEERPAR_HISTCNT
#define PEERPAR_HISTCNT 3
#endif /* !PEERPAR_HISTCNT */

struct mfloat {
	int cnt;
	float vals[PEERPAR_HISTCNT];
	float currval;
};

struct peerpar {
	struct mfloat mfloatv[PEERPAR_MAXFLOAT];
	int intv[PEERPAR_MAXINT];
	time_t timev[PEERPAR_MAXTIME];
};

enum np_type {
	PEERPAR_TYPE_FLOAT,
	PEERPAR_TYPE_INT,
	PEERPAR_TYPE_TIME,
};

enum np_valoptype {
	PEERPAR_VALOP_SINGLE,
	PEERPAR_VALOP_MED,
	PEERPAR_VALOP_AVG,
	PEERPAR_VALOP_MAX,
	PEERPAR_VALOP_MIN,
};

struct np_parvals {
	enum np_type type;
	enum np_valoptype valop;
	int idx;
	char *name;
};

enum np_cmpop {
	PEERPAR_CMPOP_EQ,
	PEERPAR_CMPOP_NE,
	PEERPAR_CMPOP_LT,
	PEERPAR_CMPOP_GT,
	PEERPAR_CMPOP_GE,
	PEERPAR_CMPOP_LE,
};

enum np_fop {
	PEERPAR_FOP_ADD,
	PEERPAR_FOP_SUB,
	PEERPAR_FOP_MULT,
	PEERPAR_FOP_DIV,
};

extern struct np_parvals peerpars[];

struct peer;

void peerpar_setfloat(struct peer *, int, float);
void peerpar_addfloat(struct peer *, int, float);
float peerpar_getfloat(const struct peer *, int);
int peerpar_getfloat2(const struct peer *, int, float *);
void peerpar_setint(struct peer *, int, int);
int peerpar_getint(const struct peer *, int);
void peerpar_settime(struct peer *, int);
time_t peerpar_gettime(const struct peer *, int);
int peerpar_adjint(struct peer *, int, int);
int peerpar_cmpfield(const struct peer *, const struct peer *, int);
int peerpar_eval(struct peer *, int, int, void *);
int peerpar_fieldop(struct peer *, int, int, int, int);
void peerpar_applyall(int, int, void *,
		      void (*)(struct peer *, int, int, void *));
void peerpar_dump(struct peer *, int);

#endif /* !_PEERPAR_H_ */
