/* peerset.h - peerset sorting, priority functions */

/*
 * Copyright (c) 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: peerset.h,v 1.8 2010/06/25 22:02:28 karls Exp $ */

#ifndef _PEERSET_H_
#define _PEERSET_H_

#include "alist.h"

#define PEERSET_MAX_PEERS 50

#define PEERSET_SHIFT_LEFT 0
#define PEERSET_SHIFT_RIGHT 1

/*XXX pset_peer() -> PSET_PEER() */
#define pset_peer(n, idx) ((struct peer *)ALIST_PVAL((n)->al_peers, (idx)))

struct peerset {
	/*
	 * the number of active peers can be reduced with pset_softtrim,
	 * meaning that the peercnt value is reduced without the entries
	 * actually being removed from the array. the fullpeercnt value
	 * contains the actual array size.
	 */
	int fullpeercnt; /* all entires in peers array */
	int peercnt; /* number of active peers in peerset */
	struct alist *al_peers;
};

struct peerset *pset_create(void);
void pset_free(struct peerset *);
struct peerset *pset_cpy(struct peerset *);
void pset_addpeer(struct peerset *, struct peer *);
int pset_delpeer(struct peerset *, struct peer *);
int pset_import(struct peerset *, int, int, void *);
void pset_softtrim(struct peerset *, int);
void pset_evalsofttrim(struct peerset *, int, int, void *);
void pset_trim(struct peerset *, int);
void pset_shift(struct peerset *, int);
void pset_sort(struct peerset *, int, int);
void pset_mysort(struct peerset *, int, int (*)(const void *, const void *));
void pset_apply(struct peerset *, int, void *,
		void (*)(struct peer *, int, void *));
void pset_applyfloat(struct peerset *, int, void *,
		   float (*)(struct peer *, int, void *));
void pset_applyint(struct peerset *, int, void *,
		   int (*)(struct peer *, int, void *));
void pset_setintrank(struct peerset *, int, int);
void pset_setfloatrank(struct peerset *, int, int);
void pset_fieldop(struct peerset *, int, int, int, int);
void pset_dumpfunc(struct peer *, int, void *);
void pset_dump(struct peerset *, int);

#endif /* _PEERSET_H_ */
