/* proc.h - defines for proc interface */

/*
 * Copyright (c) 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: proc.h,v 1.6 2010/06/25 22:02:28 karls Exp $ */

#ifndef _PROC_H_
#define _PROC_H_

/*
 * start size of handler list, ideally roughly equal to the expected
 * number of child processes that will be run.
 */
#define PROC_PROCS 10

/*
 * max number of child deaths that will be accepted before subsystem shutdown
 */
#define PROC_FATALITY_MAX 2

/* names need to be synced with proc_systnames */
enum proc_subsyst {
	PROC_RESOLVE = 0, /* first value needs to start at zero */
	PROC_MCONN,
	PROC_MAX,
};

enum proc_status {
	PROC_STATUS_NOPROC, /* no process running in slot */
	PROC_STATUS_FREE,   /* process is available for use */
	PROC_STATUS_ACTIVE, /* process is in use */
	PROC_STATUS_DEAD,   /* process died */
};

struct proc {
	pid_t pid;
	int fd;
	enum proc_status status; /* not running, free, busy, etc. */
	/* following redundant in some contexts, but make interfaces easier */
	enum proc_subsyst subsyst;
	int idx; /* subsyst proc number */
	int killed; /* killed (and descriptor closed) */
};

enum proc_type {
	PROC_TYPE_FIXED, /* always have fixed number of processes */
	PROC_TYPE_GROW,  /* grow up to max number of processes if loaded */
};

extern int proc_enabled[PROC_MAX];

void proc_preinit(void);
void proc_init(enum proc_subsyst, int, int, int, enum proc_type,
	       void (*)(int));
void proc_enqueue(enum proc_subsyst, void *, void (*)(void *, struct proc *),
		  void (*)(void *));
int proc_queuesize(enum proc_subsyst);
void proc_returnproc(struct proc *);
void proc_kill(struct proc *);
void proc_checksig(void);
void proc_shutdown(enum proc_subsyst);
void proc_dump(void);

/* internal */
void proc_sigchld(int);
struct proc *proc_pid2proc(pid_t);
struct proc *proc_getproc(enum proc_subsyst);
void proc_submitjob(enum proc_subsyst);
int proc_fork(enum proc_subsyst, int);
void proc_refork(struct proc *);

#endif /* _PROC_H_ */
