/* pset_addr.c - functions for adding resolve addrinfo to peer */

/*
 * Copyright (c) 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: pset_addr.c,v 1.4 2010/06/25 22:02:28 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <sys/types.h>
#include <sys/socket.h>

#include <netdb.h>
#include <string.h>

#include "log.h"
#include "peer.h"
#include "peerset.h"
#include "pset_addr.h"


/* pset_addr2pset - create peerset from addrinfo, adding any new peers */
struct peerset *
pset_addr2pset(struct addrinfo *addr)
{
	const char *f = "pset_addr2pset";
	char name[NI_MAXHOST];
	char port[NI_MAXSERV];
	struct addrinfo *res;
	struct peerset *pset;
	int n;

	if (addr == NULL)
		return NULL;

	pset = pset_create();
	res = addr;
	do {
		if ((n = getnameinfo(res->ai_addr, res->ai_addrlen,
				     name, sizeof(name), port, sizeof(port),
				     NI_NUMERICHOST|NI_NUMERICSERV)) != 0) {
			swarn("%s: getnameinfo: %s:%s: %s", f, name, port,
			      gai_strerror(n));
			/* should not happen */
			ERR("getnameinfo failures on addrinfo");
			continue;
		}
		pset_addpeer(pset, peer_add(name, port));
	} while ((res = res->ai_next) != NULL);

	ASSERT(pset->peercnt != 0, "no entries added from addrinfo");

	return pset;
}


/* pset_pset2addr - return addrinfo struct containing peerset entries */
struct addrinfo *
pset_pset2addr(struct peerset *pset)
{
	struct addrinfo **rp, *res, *res0;
	int i;

	rp = &res0;
	for (i = 0; i < pset->peercnt; i++) {
		res = addrinfo_dup(pset_peer(pset, i)->addr);
		ASSERT(res->ai_next == NULL, "multiple addrinfo entries");

		*rp = res;
		rp = &res->ai_next;
	}
	*rp = NULL;

	return res0;
}


/* addrinfo_dup - duplicate interesting parts of addrinfo */
struct addrinfo *
addrinfo_dup(struct addrinfo *addr)
{
	struct addrinfo **rp, *res, *res0;

	if (addr == NULL)
		return NULL;

	rp = &res0;
	do {
		/* ensure all data is freeable with one free() call */
		res = xmalloc(sizeof(struct addrinfo) + addr->ai_addrlen);
		bcopy(addr, res, sizeof(struct addrinfo));
		res->ai_next = NULL;
		res->ai_addr = (struct sockaddr *)(res + 1);
		bcopy(addr->ai_addr, res->ai_addr, addr->ai_addrlen);

		*rp = res;
		rp = &res->ai_next;
	} while ((addr = addr->ai_next) != NULL);
	/* set ai_next of last entry to NULL */
	*rp = NULL;

	return res0;
}
