/* socket.c - sockets api functions */

/*
 * Copyright (c) 2008, 2009, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: socket.c,v 1.9 2010/06/30 08:37:59 karls Exp $ */

#ifdef HAVE_CONFIG_H
#include "autoconf.h"
#endif /* HAVE_CONFIG_H */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>

#include <netinet/in.h>
#include <netinet/tcp.h>

#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "log.h"
#include "socket.h"


/* socketprep - set socket options for new socket */
int
socketprep(int s)
{
	const char *f = "socketprep";
	socklen_t optlen;
	int flags;
	int opt;

	if ((flags = fcntl(s, F_GETFL, 0)) == -1)
		ERR("fcntl");
	if (fcntl(s, F_SETFL, flags | O_NONBLOCK) == -1)
		ERR("fcntl");

	optlen = sizeof(opt);
#ifdef TCP_SACK_ENABLE
	opt = 1;
	if (setsockopt(s, IPPROTO_TCP, TCP_SACK_ENABLE, &opt, optlen) == -1)
		swarn("%s: setsockopt TCP_SACK_ENABLE: %m", f);
#endif /* TCP_SACK_ENABLE */

	opt = 1;
	if (setsockopt(s, IPPROTO_TCP, TCP_NODELAY, &opt, optlen) == -1)
		swarn("%s: setsockopt TCP_NODLAY: %m", f);

#if 0
	opt = SOCKET_BUFSIZE;
	if (setsockopt(s, SOL_SOCKET, SO_SNDBUF, &opt, optlen) == -1)
		swarn("%s: setsockopt: %m", f);

	opt = SOCKET_BUFSIZE;
	if (setsockopt(s, SOL_SOCKET, SO_RCVBUF, &opt, optlen) == -1)
		swarn("%s: setsockopt: %m", f);
#endif

	return 0;
}


/* socketerr - return socket error if any, zero otherwise */
int
socketerr(int s)
{
	const char *f = "socketerr";
	socklen_t optlen;
	int error;

	optlen = sizeof(error);
	if (getsockopt(s, SOL_SOCKET, SO_ERROR, &error, &optlen) == -1) {
		/* handle solaris returning -1 */
		dbg("%s: getsockopt failed, assuming socket error", f);
		error = errno;
	}
	if (error) {
		errno = error; /* allow error to be accessed by caller */
		dbg("%s: %m", f);
		return -1;
	} else {
/*		dbg("%s: socket ok", f);*/
		return 0;
	}
}
