/* stream.h - stream related defines, structures and function prototypes */

/*
 * Copyright (c) 2007, 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: stream.h,v 1.32 2010/08/02 22:24:56 karls Exp $ */

#ifndef _STREAM_H_
#define _STREAM_H_

#define NETWORK_IOSIZE (64 * 1024)

#if 0
/* disable checksum calculation (e.g. for stream requests from local cache) */
#define STREAM_NOCKSUM 32
#endif

#define stream_availbytes(s) (urient_availbytes((s)->ue, stream_curroff(s)))

#define stream_availblock(stream) \
	(urient_availblock((stream)->ue, (stream)->bnum, (stream)->blockoff))

enum streamdeadline {
	STREAM_NODEADLINE, /* stream without transfer deadline */
	STREAM_DEADLINE,   /* stream with transfer deadline */
};

enum streamtype {
	STREAM_FROM_MEM,
	STREAM_TO_FILE,
};

enum streamevent {
	STREAMEVENT_STREAM_START = 1,
	STREAMEVENT_STREAM_TOOSLOW,
	STREAMEVENT_STREAM_STOPPED,
	STREAMEVENT_STREAM_FAIL,
	STREAMEVENT_STREAM_COMPLETE,
	STREAMEVENT_STREAM_END,
	STREAMEVENT_STREAM_WROTEDATA,
	STREAMEVENT_STREAM_READDATA,
	STREAMEVENT_STREAM_SENTBLOCK,
};

enum streamstate {
	STREAM_ACTIVE,
	STREAM_STOPPED,
};

struct stream {
	enum streamtype type;
	enum streamstate state;
	int sfd; /* descriptor to source */
	int dfd; /* descriptor to destination */
	struct urient *ue;
	struct connection *conn; /*XXX point back to connection*/
	int32_t bnum; /* current block number */
	uint32_t blockoff; /* current offset (from start of block) */
	off_t startoff; /* absolute offset of range start (from file start) */
	off_t endoff; /* absolute offset of range end (from file start) */
	size_t readsize; /* buffer size used for reading */
	size_t writesize; /* buffer size used for writing */
	off_t nbytes; /* number of bytes read/written */
	int sendfile; /* use sendfile() to send data */
	struct timeval ct; /* time of creation */
	struct timeval blockstart; /* start time for current block */
	struct timeval stoptime; /* time of stream being stopped */
};

struct block;

struct stream *stream_new(enum streamtype, int, int, struct urient *, off_t,
	off_t);
void stream_sendstart(enum streamdeadline, struct connection *, struct urient *,
	off_t, off_t);
void stream_recvstart(enum streamdeadline, struct connection *, struct urient *,
	off_t, off_t);
void stream_end(struct stream *);
void stream_free(struct stream *);
void stream_wake(struct stream *);
void stream_blockinit(struct connection *, int32_t);
void stream_blockend(struct connection *, int32_t);
void stream_send(struct connection *);
void stream_recv(struct connection *);
void stream_stop(struct stream *);
void stream_restart(struct stream *);
void stream_tooslow(struct connection *);
off_t stream_curroff(struct stream *);
uint32_t stream_blockendoff(struct stream *, struct block *);
int stream_complete(struct stream *);
char *stream_type2str(enum streamtype);
char *stream_state2str(enum streamstate);
char *conn_streamev2str(enum streamevent);

#endif /* _STREAM_H_ */
