/* tcpmodel.h - constants for tcp model */

/*
 * Copyright (c) 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: tcpmodel.h,v 1.5 2010/10/11 09:07:47 karls Exp $ */

#define RTT_LOSSP_SQRT 0.1 /* sqrt(0.01) */ /* 'good' quality */
#define RTT_N 4 /* how much slower than estimate we allow */
#define RTT_MINLAN 10 /* upper limit for estimate */

#define rttminmbps(p) (rttmbps(p)/RTT_N)

/*
 * set minbps for transfers to bps, divided by below ratio.
 * treat peers with rtt below this value as on the same LAN
 * XXX RTT_MINLAN similar purpose but different value, needs tuning
 */
#define PEER_LAN_RTTLIM 3
#define PEER_MINBPS_LANRATIO 10
#define PEER_MINBPS_WANRATIO 3

#define PEER_MINSPEED 0.08 /* mbps */

float tcpmodel(float, int);
float rttmbps(struct peer *);
float peer_rate(struct peer *);
float peer_xfrdeadline(struct peer *, off_t);
void peer_tooslow(struct peer *, off_t, float);
void peer_addrtt(struct peer *, float);
void peer_addrate(struct peer *, float, off_t);
int peerdiv(struct peer *);
float rate_len2mbps(float, off_t);
