/* uri.h - URI parsing function defines and structures */

/*
 * Copyright (c) 2007, 2008, 2009, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: uri.h,v 1.28 2010/10/06 08:17:24 karls Exp $ */

#ifndef _URI_H_
#define _URI_H_

#include <netdb.h>

#define SCHEME_MAXLEN 10

#define URI_MAXLEN 2048 /* length includes trailing '\0' */

/* uri struct, contains decoded and normalized elements */
struct uri {
	/* normalized uri (should be fairly unique, but aliasing possible) */
	char uri[URI_MAXLEN];
	/* uri elements */
	char scheme[SCHEME_MAXLEN]; /* can be different from scheme in uri */
	char host[NI_MAXHOST];
	char port[NI_MAXSERV];
	char path[PATH_MAX];
	/* XXXhave encoded variants of host/path? */
};

/**
 * uri_parse - encode text string uri for transmission
 *
 * A pointer to a static buffer with the encoded uri is returned.
 */
char *
uri_encode(char *);

/**
 * uri_parse - parse URI into struct uri
 *
 * Given an URI in text format, this function will verify, decode, and
 * divide it into its various separate elements.
 *
 * -the uri.uri value will contain a normalized version of the input
 * -the scheme is copied from the original value (e.g., http)
 * -the path will not be sanitized; handling of potentially dangerous
 *  elements is the responsibility of the caller
 */
int
uri_parse(struct uri *, const char *);

int
uri_joinparts(struct uri *);

/* internal */
int uri_unescape(char *, const char *, size_t)
	__attribute__((__bounded__(__string__, 1, 3)));

#endif /* _URI_H_ */
