/* urient.h - defines, required includes, structures and prototypes */

/*
 * Copyright (c) 2007, 2008, 2010
 *      Inferno Nettverk A/S, Norway.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. The above copyright notice, this list of conditions and the following
 *    disclaimer must appear in all copies of the software, derivative works
 *    or modified versions, and any portions thereof, aswell as in all
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by
 *      Inferno Nettverk A/S, Norway.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Inferno Nettverk A/S requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  info@inet.no
 *  Inferno Nettverk A/S
 *  Oslo Research Park
 *  Gaustadalleen 21
 *  NO-0349 Oslo
 *  Norway
 *
 * any improvements or extensions that they make and grant Inferno Nettverk A/S
 * the rights to redistribute these changes.
 *
 */

/* $Id: urient.h,v 1.21 2010/06/25 22:02:29 karls Exp $ */

#ifndef _URIENT_H_
#define _URIENT_H_

#include "cachefile.h"
#include "filemeta.h"
#include "uri.h"

#define URIENT_ENCSTR_MAX 4096 /*XXX*/

#define URIENT_MAX_ENTRIES 1024 /*XXX dyn?*/

enum uestate {
	URIENT_STATE_NONE = 0,
	URIENT_STATE_URI,        /* cache */
	URIENT_STATE_META,       /* cache */
	URIENT_STATE_INCOMPLETE, /* cache */
	URIENT_STATE_COMPLETE,   /* cache|server */
};

struct ueops {
	void (*uldat_free)(void *);
	void (*cacheinit)(void *);
};

struct urient {
	struct uri uri;
	enum uestate state;
	struct filemeta *meta;
	char *prefix; /* pointer to pubdir/cachedir */
	char filename[FILENAME_MAX]; /* disk file name (not full path) */
	int master; /* true if this is master copy of file (not cached) */
	struct cachefile *cf; /* cachefile, set if file is cached */
	struct file *file;
	time_t lastreq; /* time of last request */
	struct ueops *ueops;
	void *uldat;
};

/*XXX*/
extern struct alist *al_urient;

void urient_preinit(void);
struct urient *urient_find(struct uri *);
struct urient *urient_add(struct uri *);
void urient_del(struct urient *);
int urient_setfile(struct urient *, char *, char *);
void urient_gotreq(struct urient *);
void urient_status(void);
int urient_decstr(struct urient *, char *);
char *urient_encstr(struct urient *);

/* urient_block.c */
int32_t urient_subseq(struct urient *, int32_t);
int32_t urient_blockcount(struct urient *, int32_t);
int urient_haveblock(struct urient *, int32_t);
off_t urient_availbytes(struct urient *, off_t);
uint32_t urient_availblock(struct urient *, int32_t, uint32_t);
int32_t off2block(struct urient *, off_t);
off_t block2off(struct urient *, int32_t);
int urient_iscomp(struct urient *);

/* urient_state.c */
struct urient *urient_servinit(struct uri *);
int urient_cacheinit(struct urient *, off_t, int32_t, char *);
int urient_fileref(struct urient *);
void urient_fileunref(struct urient *);
void urient_gotblock(struct urient *, int);
void urient_lostblock(struct urient *, int);
void urient_purgeblock(struct urient *, int);

#endif /* _URIENT_H_ */
